package net.wanji.common.action;

import net.wanji.common.annotation.ActionBean;
import net.wanji.common.annotation.ActionMethod;
import net.wanji.common.annotation.ActionMethodParameter;
import net.wanji.common.enums.Datatype;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import sun.reflect.generics.reflectiveObjects.ParameterizedTypeImpl;

import java.lang.reflect.AnnotatedType;
import java.lang.reflect.Type;
import java.math.BigDecimal;
import java.util.*;

/**
 * <>
 *
 * @author wanji
 * 2020/6/21 0:33
 */
@Slf4j
public class BuiltInActionLibraryUtils {

    private static List<SpringBean> springBeans;

    private static List<Class<?>> builtInClass;

    static {
        springBeans = Lists.newArrayList();
        builtInClass = Lists.newArrayList();

        builtInClass.add(DateAction.class);
        builtInClass.add(StringAction.class);
    }

    /**
     * 获取内置方法
     *
     * @return List
     */
    public static List<SpringBean> getBuiltInActions() {
        return BuiltInActionLibraryUtils.springBeans;
    }

    public static void init() {
        BuiltInActionLibraryUtils.springBeans.clear();
        for (Class<?> clazz : BuiltInActionLibraryUtils.builtInClass) {
            ActionBean annotation = clazz.getAnnotation(ActionBean.class);
            if (Objects.isNull(annotation)) {
                return;
            }
            SpringBean springBean = new SpringBean();
            springBean.setName(clazz.getSimpleName());
            springBean.setLabel(annotation.name());
            springBean.setPath(clazz.getName());
            springBean.setMethods(buildMethods(clazz.getMethods()));

            BuiltInActionLibraryUtils.springBeans.add(springBean);
        }
    }

    /**
     * 构建内置方法
     *
     * @param methods 反射的方法数组
     * @return List
     */
    private static List<Method> buildMethods(java.lang.reflect.Method[] methods) {
        List<Method> methodList = org.apache.commons.compress.utils.Lists.newArrayList();
        for (java.lang.reflect.Method method : methods) {
            ActionMethod methodAnnotation = method.getAnnotation(ActionMethod.class);
            if (Objects.isNull(methodAnnotation)) {
                continue;
            }
            //返回值类型
            AnnotatedType annotatedReturnType = method.getAnnotatedReturnType();
            String methodName = method.getName();
            Method builtInMethod = new Method();
            builtInMethod.setMethodName(methodName);
            builtInMethod.setLabel(methodAnnotation.name());
            if (Objects.nonNull(annotatedReturnType)) {
                try {
                    Type type = annotatedReturnType.getType();
                    Datatype datatype = Datatype.Object;
                    if (type instanceof Class) {
                        datatype = buildDatatype((Class<?>) type);
                    } else if (type instanceof ParameterizedTypeImpl) {
                        datatype = buildDatatype(((ParameterizedTypeImpl) type).getRawType());
                    }
                    builtInMethod.setDatatype(datatype.name());
                } catch (Exception e) {
                    log.error("初始化内置方法异常", e);
                }
            }
            methodList.add(builtInMethod);
            ActionMethodParameter methodParam = method.getAnnotation(ActionMethodParameter.class);
            List<String> parameterNames = org.apache.commons.compress.utils.Lists.newArrayList();
            if (Objects.nonNull(methodParam)) {
                String[] paramNames = methodParam.names();
                Collections.addAll(parameterNames, paramNames);
            }
            builtInMethod.setParameters(buildParameters(method, parameterNames));
        }
        return methodList;
    }

    /**
     * 构建内置方法的参数集合
     *
     * @param method         反射的方法
     * @param parameterNames 参数名
     * @return
     */
    private static List<Parameter> buildParameters(java.lang.reflect.Method method, List<String> parameterNames) {
        List<Parameter> parameters = org.apache.commons.compress.utils.Lists.newArrayList();

        Class<?>[] paramClasses = method.getParameterTypes();
        for (int i = 0; i < paramClasses.length; i++) {
            Class<?> paramClass = paramClasses[i];
            String name = "";
            if (parameterNames.size() > i) {
                name = parameterNames.get(i);
            }
            Parameter parameter = new Parameter();
            parameter.setLabel(name);
            parameter.setDatatype(buildDatatype(paramClass).name());
            parameters.add(parameter);
        }
        return parameters;
    }

    /**
     * 构建数据类型
     *
     * @param clazz
     * @return Datatype
     */
    private static Datatype buildDatatype(Class<?> clazz) {
        if ("java.lang.Object".equals(clazz.getName())) {
            return Datatype.Object;
        }
        if (clazz.isAssignableFrom(Integer.class) || clazz.isAssignableFrom(int.class)) {
            return Datatype.Integer;
        } else if (clazz.isAssignableFrom(Long.class) || clazz.isAssignableFrom(long.class)) {
            return Datatype.Long;
        } else if (clazz.isAssignableFrom(Double.class) || clazz.isAssignableFrom(double.class)) {
            return Datatype.Double;
        } else if (clazz.isAssignableFrom(Float.class) || clazz.isAssignableFrom(float.class)) {
            return Datatype.Float;
        } else if (clazz.isAssignableFrom(BigDecimal.class)) {
            return Datatype.BigDecimal;
        } else if (clazz.isAssignableFrom(Boolean.class) || clazz.isAssignableFrom(boolean.class)) {
            return Datatype.Boolean;
        } else if (clazz.isAssignableFrom(Date.class)) {
            return Datatype.Date;
        } else if (clazz.isAssignableFrom(List.class)) {
            return Datatype.List;
        } else if (clazz.isAssignableFrom(Set.class)) {
            return Datatype.Set;
        } else if (clazz.isAssignableFrom(Enum.class)) {
            return Datatype.Enum;
        } else if (clazz.isAssignableFrom(Map.class)) {
            return Datatype.Map;
        } else if (clazz.isAssignableFrom(String.class)) {
            return Datatype.String;
        } else if (clazz.isAssignableFrom(Character.class) || clazz.isAssignableFrom(char.class)) {
            return Datatype.Char;
        } else {
            return Datatype.Object;
        }
    }

}
