package net.wanji.common.framework.db.impl;

import net.wanji.common.framework.Constants;
import net.wanji.common.framework.db.DbInterface;
import net.wanji.common.framework.domain.Db;
import net.wanji.common.framework.exception.NodeException;
import net.wanji.common.framework.rest.Page;
import net.wanji.common.tool.dbtools.MySQL;
import lombok.extern.slf4j.Slf4j;

import java.util.List;
import java.util.Map;

import static net.wanji.common.framework.Constants.DbDriver.MYSQL;
import static net.wanji.common.framework.Constants.MysqlColumn.*;
import static net.wanji.common.framework.Constants.ServicePublishConstants.SqlConstants.BLANK;
import static net.wanji.common.framework.Constants.ServicePublishConstants.SqlConstants.FIELD_NAME_EN;

/**
 * Mysql数据库的基础操作接口实现
 *
 * @author wanji
 */
@Slf4j
public class MysqlDBImpl implements DbInterface {

    @Override
    public String getDriver() {
        return Constants.DbDriver.MYSQL;
    }

    @Override
    public String getDBUrl(Db dataBase) {
        return new StringBuilder("jdbc:mysql://")
                .append(dataBase.getIp()).append(":")
                .append(dataBase.getPort()).append("/")
                .append(dataBase.getName())
                .append("?useUnicode=true&characterEncoding=UTF-8").toString();
    }

    @Override
    public String getInsertSQL(String tableName, List<Map<String, Object>> insertParams) {
        // 拼接带有占位符的insertSql: insert into table (col1, col2...) values (val1, val2...)
        StringBuilder columns = new StringBuilder("(");
        StringBuilder values = new StringBuilder("(");
        int paramSize = insertParams.size();
        for (int i = 0; i < paramSize; i++) {
            Map<String, Object> insertParam = insertParams.get(i);
            String fieldName = insertParam.get(FIELD_NAME_EN).toString();
            if (i == (paramSize - 1)) {
                columns.append(fieldName).append(")");
                values.append("?)");
            } else {
                columns.append(fieldName).append(",");
                values.append("?,");
            }
        }
        return new StringBuilder("INSERT INTO").append(BLANK)
                .append(tableName).append(BLANK)
                .append(columns).append(BLANK)
                .append("VALUES").append(BLANK)
                .append(values).toString();
    }

    @Override
    public String getUpdateSQL(String tableName, List<Map<String, Object>> updateParams,
                                  List<Map<String, Object>> whereParams) {
        return new MySQL.SqlBuilder()
                .update(tableName)
                .buildSet(updateParams)
                .buildWhere(whereParams)
                .build()
                .toString();
    }

    @Override
    public String getDeleteSQL(String tableName, List<Map<String, Object>> whereParams) {
        return new MySQL.SqlBuilder()
                .delete()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .build().toString();
    }

    @Override
    public String getFindByIdSQL(String tableName, String id) {
        return new MySQL.SqlBuilder()
                .select()
                .buildAllColumns()
                .buildFrom(tableName)
                .build()
                .toString() + " where " + id + " = ? ";
    }

    @Override
    public String getFindByWhereSQL(String tableName,
                                       List<Map<String, Object>> whereParams,
                                       List<Map<String, String>> orderMap) {
        return new MySQL.SqlBuilder()
                .select()
                .buildAllColumns()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .buildOrder(orderMap)
                .build()
                .toString();
    }

    @Override
    public String getFindCountSQL(String tableName, List<Map<String, Object>> whereParams) {
        return new MySQL.SqlBuilder()
                .select()
                .buildCount()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .build()
                .toString();
    }

    @Override
    public String getFindByPageSQL(String tableName, Page page,
                                      List<Map<String, Object>> whereParams,
                                      int startRowNum,
                                      List<Map<String, String>> orderMap) {
        return new MySQL.SqlBuilder()
                .select()
                .buildAllColumns()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .buildOrder(orderMap)
                .buildLimit(startRowNum, page.getPageSize())
                .build()
                .toString();
    }

    @Override
    public String getFindPageBySqlSQL(Page page, int startRowNum, String sql) {
        String select = new MySQL.SqlBuilder()
                .select()
                .buildAllColumns()
                .build()
                .toString();
        String limit = new MySQL.SqlBuilder()
                .buildLimit(startRowNum, page.getPageSize())
                .build()
                .toString();
        return select + " " + sql + " " + limit;
    }

    @Override
    public String getFindCountBySqlSQL(String sql) {
        String count = new MySQL.SqlBuilder()
                .select()
                .buildCount()
                .build()
                .toString();
        return count + " " + sql;
    }

    /**
     * 确定将要替代sql中占位符的实际值的类型。
     *
     * @throws NodeException 如果是未知MySQL类型。
     */
    @Override
    public Object typeHandler(String fieldType, String fieldValue)  throws NodeException {
        switch (fieldType) {
            case DATE:
            case YEAR:
            case DATETIME:
            case TIMESTAMP:
            case TEXT:
            case LONGTEXT:
            case VARCHAR:
            case CHAR:
                return fieldValue;
            case INT:
            case TINYINT:
            case SMALLINT:
            case MEDIUMINT:
                return Integer.valueOf(fieldValue);
            case BIT:
                return Boolean.valueOf(fieldValue);
            case FLOAT:
                return Float.valueOf(fieldValue);
            case DOUBLE:
                return Double.valueOf(fieldValue);
            default:
                throw new NodeException("[MysqlTypeHandler] 未支持的类型" + fieldType);
        }
    }

}
