package net.wanji.common.framework.db.impl;

import net.wanji.common.framework.Constants;
import net.wanji.common.framework.db.DbInterface;
import net.wanji.common.framework.domain.Db;
import net.wanji.common.framework.exception.NodeException;
import net.wanji.common.framework.rest.Page;
import net.wanji.common.tool.dbtools.Oracle;
import lombok.extern.slf4j.Slf4j;

import java.sql.Timestamp;
import java.util.List;
import java.util.Map;

import static net.wanji.common.framework.Constants.DbDriver.ORACLE;
import static net.wanji.common.framework.Constants.OracleColumn.*;
import static net.wanji.common.framework.Constants.ServicePublishConstants.SqlConstants.BLANK;
import static net.wanji.common.framework.Constants.ServicePublishConstants.SqlConstants.FIELD_NAME_EN;

/**
 * Oracle数据库的基础操作接口实现
 *
 * @author wanji
 */
@Slf4j
public class OracleDBImpl implements DbInterface {

    @Override
    public String getDriver() {
        return Constants.DbDriver.ORACLE;
    }

    @Override
    public String getDBUrl(Db dataBase) {
        return new StringBuilder("jdbc:oracle:thin:@")
                .append(dataBase.getIp()).append(":")
                .append(dataBase.getPort()).append(":")
                .append(dataBase.getName())
                .toString();
    }

    @Override
    public String getInsertSQL(String tableName, List<Map<String, Object>> insertParams) {
        // 2、拼接带有占位符的insertSql
        StringBuilder columns = new StringBuilder("(");
        StringBuilder values = new StringBuilder("(");
        int paramSize = insertParams.size();
        for (int i = 0; i < paramSize; i++) {
            Map<String, Object> insertParam = insertParams.get(i);
            String fieldName = insertParam.get(FIELD_NAME_EN).toString();
            if (i == (paramSize - 1)) {
                columns.append("\"").append(fieldName).append("\"").append(")");
                values.append("?)");
            } else {
                columns.append("\"").append(fieldName).append("\"").append(",");
                values.append("?,");
            }
        }
        return new StringBuilder("INSERT INTO").append(BLANK)
                .append(tableName).append(BLANK)
                .append(columns).append(BLANK)
                .append("VALUES").append(BLANK)
                .append(values).toString();
    }

    @Override
    public String getUpdateSQL(String tableName, List<Map<String, Object>> updateParams,
                                  List<Map<String, Object>> whereParams) {
        return new Oracle.SqlBuilder()
                .update(tableName)
                .buildSet(updateParams)
                .buildWhere(whereParams)
                .build()
                .toString();
    }

    @Override
    public String getDeleteSQL(String tableName, List<Map<String, Object>> whereParams) {
        return new Oracle.SqlBuilder()
                .delete()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .build().toString();
    }

    @Override
    public String getFindByIdSQL(String tableName, String id) {
        return new Oracle.SqlBuilder()
                .select()
                .buildAllColumns()
                .buildFrom(tableName)
                .build()
                .toString() + " WHERE \"" + id + "\" = ? ";
    }

    @Override
    public String getFindByWhereSQL(String tableName,
                                       List<Map<String, Object>> whereParams,
                                       List<Map<String, String>> orderMap) {
        return new Oracle.SqlBuilder()
                .select()
                .buildAllColumns()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .buildOrder(orderMap)
                .build()
                .toString();
    }

    @Override
    public String getFindCountSQL(String tableName, List<Map<String, Object>> whereParams) {
        return new Oracle.SqlBuilder()
                .select()
                .buildCount()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .build()
                .toString();
    }

    @Override
    public String getFindByPageSQL(String tableName, Page page,
                                      List<Map<String, Object>> whereParams,
                                      int startRowNum,
                                      List<Map<String, String>> orderMap) {
        String prefixSql = "SELECT * FROM ( SELECT all_rows.*, ROWNUM RN FROM (";
        String infixSelectSql = new Oracle.SqlBuilder()
                .select()
                .buildAllColumns()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .buildOrder(orderMap)
                .build()
                .toString();
        int endRowNum = page.getPageSize() + startRowNum;
        String postfixSql = ") all_rows WHERE ROWNUM <= " + endRowNum + ") WHERE RN > " + startRowNum;
        return prefixSql + infixSelectSql + postfixSql;
    }

    @Override
    public String getFindPageBySqlSQL(Page page, int startRowNum, String sql) {
        String select = new Oracle.SqlBuilder()
                .select()
                .buildAllColumns()
                .build()
                .toString();

        int endRowNum = page.getPageSize() + startRowNum;
        String postfixSql = ") all_rows WHERE ROWNUM <= " + endRowNum + ") WHERE RN > " + startRowNum;

        return select + " " + sql + " " + postfixSql;
    }

    @Override
    public String getFindCountBySqlSQL(String sql) {
        String count = new Oracle.SqlBuilder()
                .select()
                .buildCount()
                .build()
                .toString();
        return count + " " + sql;
    }

    /**
     * 确定将要替代sql中占位符的实际值的类型。
     *
     * @throws NodeException 如果是未知Oracle类型。
     */
    public Object typeHandler(String fieldType, String fieldValue) throws NodeException {
        switch (fieldType) {
            case ORACLE_CHAR:
            case VARCHAR2:
            case VARCHAR:
                return fieldValue;
            case NUMBER:
                return Long.valueOf(fieldValue);
            case FLOAT:
                return Double.valueOf(fieldValue);
            case TIMESTAMP:
            case DATE:
                return Timestamp.valueOf(fieldValue);
            default:
                throw new NodeException("[OracleTypeHandler] 未支持的类型" + fieldType);
        }
    }
}
