package net.wanji.common.framework.db.impl;

import net.wanji.common.framework.Constants;
import net.wanji.common.framework.db.DbInterface;
import net.wanji.common.framework.domain.Db;
import net.wanji.common.framework.exception.NodeException;
import net.wanji.common.framework.rest.Page;
import net.wanji.common.tool.dbtools.PostgreSQL;
import lombok.extern.slf4j.Slf4j;

import java.math.BigDecimal;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.List;
import java.util.Map;

import static net.wanji.common.framework.Constants.PostGreSqlColumn.*;
import static net.wanji.common.framework.Constants.ServicePublishConstants.SqlConstants.BLANK;
import static net.wanji.common.framework.Constants.ServicePublishConstants.SqlConstants.FIELD_NAME_EN;

/**
 * Mysql数据库的基础操作接口实现
 *
 * @author wanji
 */
@Slf4j
public class PostgresqlDBImpl implements DbInterface {

    @Override
    public String getDriver() {
        return Constants.DbDriver.POSTGRESQL;
    }

    @Override
    public String getDBUrl(Db dataBase) {
        return new StringBuilder("jdbc:postgresql://")
                .append(dataBase.getIp()).append(":")
                .append(dataBase.getPort()).append("/")
                .append(dataBase.getName())
                .toString();
    }

    @Override
    public String getInsertSQL(String tableName, List<Map<String, Object>> insertParams) {
        // 拼接带有占位符的insertSql: insert into table (col1, col2...) values (val1, val2...)
        StringBuilder columns = new StringBuilder("(");
        StringBuilder values = new StringBuilder("(");
        int paramSize = insertParams.size();
        for (int i = 0; i < paramSize; i++) {
            Map<String, Object> insertParam = insertParams.get(i);
            String fieldName = insertParam.get(FIELD_NAME_EN).toString();
            if (i == (paramSize - 1)) {
                columns.append(fieldName).append(")");
                values.append("?)");
            } else {
                columns.append(fieldName).append(",");
                values.append("?,");
            }
        }
        return new StringBuilder("INSERT INTO").append(BLANK)
                .append(tableName).append(BLANK)
                .append(columns).append(BLANK)
                .append("VALUES").append(BLANK)
                .append(values).toString();
    }

    @Override
    public String getUpdateSQL(String tableName, List<Map<String, Object>> updateParams,
                                  List<Map<String, Object>> whereParams) {
        return new PostgreSQL.SqlBuilder()
                .update(tableName)
                .buildSet(updateParams)
                .buildWhere(whereParams)
                .build()
                .toString();
    }

    @Override
    public String getDeleteSQL(String tableName, List<Map<String, Object>> whereParams) {
        return new PostgreSQL.SqlBuilder()
                .delete()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .build().toString();
    }

    @Override
    public String getFindByIdSQL(String tableName, String id) {
        return new PostgreSQL.SqlBuilder()
                .select()
                .buildAllColumns()
                .buildFrom(tableName)
                .build()
                .toString() + " where " + id + " = ? ";
    }

    @Override
    public String getFindByWhereSQL(String tableName,
                                       List<Map<String, Object>> whereParams,
                                       List<Map<String, String>> orderMap) {
        return new PostgreSQL.SqlBuilder()
                .select()
                .buildAllColumns()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .buildOrder(orderMap)
                .build()
                .toString();
    }

    @Override
    public String getFindCountSQL(String tableName, List<Map<String, Object>> whereParams) {
        return new PostgreSQL.SqlBuilder()
                .select()
                .buildCount()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .build()
                .toString();
    }

    @Override
    public String getFindByPageSQL(String tableName, Page page,
                                      List<Map<String, Object>> whereParams,
                                      int startRowNum,
                                      List<Map<String, String>> orderMap) {
        return new PostgreSQL.SqlBuilder()
                .select()
                .buildAllColumns()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .buildOrder(orderMap)
                .buildLimit(startRowNum, page.getPageSize())
                .build()
                .toString();
    }

    @Override
    public String getFindPageBySqlSQL(Page page, int startRowNum, String sql) {
        String select = new PostgreSQL.SqlBuilder()
                .select()
                .buildAllColumns()
                .build()
                .toString();
        String limit = new PostgreSQL.SqlBuilder()
                .buildLimit(startRowNum, page.getPageSize())
                .build()
                .toString();
        return select + " " + sql + " " + limit;
    }

    @Override
    public String getFindCountBySqlSQL(String sql) {
        String count = new PostgreSQL.SqlBuilder()
                .select()
                .buildCount()
                .build()
                .toString();
        return count + " " + sql;
    }

    /**
     * 确定将要替代sql中占位符的实际值的类型。
     *
     * @throws NodeException 如果是未知 Beyon 类型。
     */
    @Override
    public Object typeHandler(String fieldType, String fieldValue) throws NodeException {
        switch (fieldType) {
            case FLOAT4:
                return Float.valueOf(fieldValue);
            case FLOAT8:
            case MONEY:
                return Double.valueOf(fieldValue);
            case INT4:
            case INT2:
                return Integer.valueOf(fieldValue);
            case BOOL:
            case net.wanji.common.framework.Constants.PostGreSqlColumn.BIT:
                return Boolean.valueOf(fieldValue);
            case NUMERIC:
                return new BigDecimal(fieldValue);
            case INT8:
                return Long.valueOf(fieldValue);
            case TIMESTAMP:
                return Timestamp.valueOf(fieldValue);
            case TIME:
                return Time.valueOf(fieldValue);
            case DATE:
                return Date.valueOf(fieldValue);
            case BYTEA:
            case TEXT:
            case BPCHAR:
            case net.wanji.common.framework.Constants.PostGreSqlColumn.VARCHAR:
            case BOX:
            case CIDR:
            case INET:
            case MACADDR:
            case VARBIT:
            case CIRCLE:
            case INTERVAL:
            case LINE:
            case LSEG:
            case POLYGON:
            case POINT:
            case PATH:
                return fieldValue;
            default:
                throw new NodeException("[BeyonTypeHandler] 未支持的类型:" + fieldType);
        }
    }
}
