package net.wanji.common.framework.db.impl;

import net.wanji.common.framework.Constants;
import net.wanji.common.framework.db.DbInterface;
import net.wanji.common.framework.domain.Db;
import net.wanji.common.framework.exception.NodeException;
import net.wanji.common.framework.rest.Page;
import net.wanji.common.tool.dbtools.SQLServer;
import lombok.extern.slf4j.Slf4j;

import java.sql.Timestamp;
import java.util.List;
import java.util.Map;

import static net.wanji.common.framework.Constants.DbDriver.SQLSERVER;
import static net.wanji.common.framework.Constants.MysqlColumn.*;
import static net.wanji.common.framework.Constants.ServicePublishConstants.SqlConstants.BLANK;
import static net.wanji.common.framework.Constants.ServicePublishConstants.SqlConstants.FIELD_NAME_EN;
import static net.wanji.common.framework.Constants.SqlServerColumn.DATETIME2;

/**
 * SQLServer 数据库的基础操作接口实现
 *
 * @author wanji
 */
@Slf4j
public class SQLServerDBImpl implements DbInterface {

    @Override
    public String getDriver() {
        return Constants.DbDriver.SQLSERVER;
    }

    @Override
    public String getDBUrl(Db dataBase) {
        return new StringBuilder("jdbc:sqlserver://")
                .append(dataBase.getIp()).append(":")
                .append(dataBase.getPort()).append(";DatabaseName=").append(dataBase.getName()).toString();
    }

    @Override
    public String getInsertSQL(String tableName, List<Map<String, Object>> insertParams) {
        // 拼接带有占位符的insertSql: insert into table (col1, col2...) values (val1, val2...)
        StringBuilder columns = new StringBuilder("(");
        StringBuilder values = new StringBuilder("(");
        int paramSize = insertParams.size();
        for (int i = 0; i < paramSize; i++) {
            Map<String, Object> insertParam = insertParams.get(i);
            String fieldName = insertParam.get(FIELD_NAME_EN).toString();
            if (i == (paramSize - 1)) {
                columns.append(fieldName).append(")");
                values.append("?)");
            } else {
                columns.append(fieldName).append(",");
                values.append("?,");
            }
        }
        // 插入的sql语句
        return new StringBuilder("INSERT INTO").append(BLANK)
                .append(tableName).append(BLANK)
                .append(columns).append(BLANK)
                .append("VALUES").append(BLANK)
                .append(values).toString();
    }

    @Override
    public String getUpdateSQL(String tableName, List<Map<String, Object>> updateParams,
                                  List<Map<String, Object>> whereParams) {
        return new SQLServer.SqlBuilder()
                .update(tableName)
                .buildSet(updateParams)
                .buildWhere(whereParams)
                .build()
                .toString();
    }

    @Override
    public String getDeleteSQL(String tableName, List<Map<String, Object>> whereParams) {
        return new SQLServer.SqlBuilder()
                .delete()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .build().toString();
    }

    @Override
    public String getFindByIdSQL(String tableName, String id) {
        return new SQLServer.SqlBuilder()
                .select()
                .buildAllColumns()
                .buildFrom(tableName)
                .build()
                .toString() + " where " + id + " = ? ";
    }

    @Override
    public String getFindByWhereSQL(String tableName,
                                       List<Map<String, Object>> whereParams,
                                       List<Map<String, String>> orderMap) {
        return new SQLServer.SqlBuilder()
                .select()
                .buildAllColumns()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .buildOrder(orderMap)
                .build()
                .toString();
    }

    @Override
    public String getFindCountSQL(String tableName, List<Map<String, Object>> whereParams) {
        return new SQLServer.SqlBuilder()
                .select()
                .buildCount()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .build()
                .toString();
    }

    @Override
    public String getFindByPageSQL(String tableName, Page page,
                                      List<Map<String, Object>> whereParams,
                                      int startRowNum,
                                      List<Map<String, String>> orderMap) {
        return new SQLServer.SqlBuilder()
                .select()
                .buildAllColumns()
                .buildFrom(tableName)
                .buildWhere(whereParams)
                .buildOrder(orderMap)
                .buildOffset(startRowNum, page.getPageSize())
                .build()
                .toString();
    }

    @Override
    public String getFindPageBySqlSQL(Page page, int startRowNum, String sql) {
        String select = new SQLServer.SqlBuilder()
                .select()
                .buildAllColumns()
                .build()
                .toString();
        String limit = new SQLServer.SqlBuilder()
                .buildOffset(startRowNum, page.getPageSize())
                .build()
                .toString();
        return select + " " + sql + " " + limit;
    }

    @Override
    public String getFindCountBySqlSQL(String sql) {
        String count = new SQLServer.SqlBuilder()
                .select()
                .buildCount()
                .build()
                .toString();
        return count + " " + sql;
    }

    /**
     * @Description: sqlServer类型转换器
     **/
    @Override
    public Object typeHandler(String fieldType, String fieldValue) throws NodeException {
        switch (fieldType) {
            case VARCHAR:
            case CHAR:
            case TEXT:
                return fieldValue;
            case INT:
            case TINYINT:
            case SMALLINT:
            case BIGINT:
                return Integer.valueOf(fieldValue);
            case FLOAT:
                return Float.valueOf(fieldValue);
            case DATETIME:
            case DATETIME2:
                return Timestamp.valueOf(fieldValue);
            default:
                throw new NodeException("SQLServer 未支持的类型" + fieldType);
        }
    }
}
