package net.wanji.common.framework.dubbointerface.impl;

import com.alibaba.fastjson.JSONObject;
import net.wanji.common.framework.Constants;
import net.wanji.common.framework.domain.OperationLog;
import net.wanji.common.framework.domain.TrackableEntity;
import net.wanji.common.framework.dubbointerface.BaseDubboInterface;
import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.common.framework.mapper.BaseInterfaceMapper;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.framework.rest.Page;
import net.wanji.common.tool.mqtool.MqUtil;
import net.wanji.common.utils.tool.IDGenerator;
import net.wanji.common.utils.tool.StringUtils;
import eu.bitwalker.useragentutils.OperatingSystem;
import eu.bitwalker.useragentutils.UserAgent;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

import static net.wanji.common.framework.Constants.LogType.OPERATION_LOG;

/**
 * @author wanji
 */
@Transactional(rollbackFor = Exception.class)
@Slf4j
public abstract class BaseDubboInterfaceImpl<Entity extends TrackableEntity> implements BaseDubboInterface<Entity> {

    public static final String EXISTS = "exists";
    public static final String TRUE = "true";
    public static final String FALSE = "false";

    @Override
    public Page<Entity> findByPage(Page<Entity> page, Map<String, Object> map) throws DubboProviderException {
        try {
            if (map.get("keyword") == null || StringUtils.isBlank((String) map.get("keyword"))) {
                map.put("keyword", (Object)null);
            }
            page.setTotal(this.getBaseInterfaceMapper().getCount(map));
            page = Page.newInstance(page.getPageSize(), page.getTotal(), page.getPageNum());
            map.put("startRowNum", page.getStartRowNum());
            map.put("pageSize", page.getPageSize());
            page.setRows(this.getBaseInterfaceMapper().findByPage(map));
        } catch (Exception e) {
            log.error("BaseDubboInterfaceImpl findByPage ", e);
            throw new DubboProviderException(e.getMessage(), e);
        }
        return page;
    }

    @Override
    public JsonViewObject save(Entity entity) throws DubboProviderException {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            entity.setId(IDGenerator.getID());
            entity.setCreateDate(new Date());
            this.getBaseInterfaceMapper().save(entity);
            jsonViewObject.success(entity.getId(), "数据保存成功！");
        } catch (Exception e) {
            jsonViewObject.fail("数据保存失败！");
            log.error("BaseDubboInterfaceImpl save ", e);
            throw new DubboProviderException(e.getMessage(), e);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject update(Entity entity) throws DubboProviderException {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
//            entity.setCreateDate(new Date());
            Integer updateCount = this.getBaseInterfaceMapper().update(entity);
            jsonViewObject.success(updateCount, "数据修改成功！");
        }  catch (Exception e) {
            jsonViewObject.fail("数据修改失败！");
            log.error("BaseDubboInterfaceImpl update ", e);
            throw new DubboProviderException(e.getMessage(), e);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject deleteById(String id) throws DubboProviderException {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            Integer deleteCount = this.getBaseInterfaceMapper().deleteById(id);
            if (deleteCount < 1) {
                jsonViewObject.fail(deleteCount, "删除数据不存在！");
            } else {
                jsonViewObject.success(deleteCount, "数据删除成功！");
            }
        } catch (Exception e) {
            jsonViewObject.fail("数据删除失败！");
            log.error("BaseDubboInterfaceImpl deleteById ", e);
            throw new DubboProviderException(e.getMessage(), e);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject deleteByIds(String[] ids) throws DubboProviderException {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            Integer deleteCount = this.getBaseInterfaceMapper().deleteByIds(Arrays.asList(ids));
            if (deleteCount < 1) {
                jsonViewObject.fail(deleteCount, "删除数据不存在！");
            } else {
                jsonViewObject.success(deleteCount, "数据删除成功！");
            }
        } catch (Exception e) {
            jsonViewObject.fail("数据删除失败！");
            log.error("BaseDubboInterfaceImpl deleteById ", e);
            throw new DubboProviderException(e.getMessage(), e);
        }
        return jsonViewObject;
    }

    @Override
    public Entity findById(String id) throws DubboProviderException {
        Entity entity;
        try {
            entity = this.getBaseInterfaceMapper().findById(id);
        } catch (Exception e) {
            log.error("BaseDubboInterfaceImpl findById ", e);
            throw new DubboProviderException(e.getMessage(), e);
        }
        return entity;
    }

    @Override
    public List<Entity> findAll() throws DubboProviderException {
        List<Entity> list;
        try {
            list = this.getBaseInterfaceMapper().findAll();
        } catch (Exception e) {
            log.error("BaseDubboInterfaceImpl findAll ", e);
            throw new DubboProviderException(e.getMessage(), e);
        }
        return list;
    }

    @Override
    public List<Entity> findByMap(Map<String, Object> map) throws DubboProviderException {
        List<Entity> list;
        try {
            list = this.getBaseInterfaceMapper().findByMap(map);
        } catch (Exception e) {
            log.error("BaseDubboInterfaceImpl findByMap", e);
            throw new DubboProviderException(e.getMessage(), e);
        }
        return list;
    }

    @Override
    public void sendOperationLog(String classNameCn,String className,String method,String username,String dbId,long lastTime,JsonViewObject jsonViewObject) {
        OperationLog opLog = new OperationLog();
        try {
            opLog.setResName(classNameCn);
            opLog.setResType(className);
            opLog.setOperationType(method);
            opLog.setOperationUser(username);
            opLog.setDbId(dbId);
            opLog.setDes(Objects.isNull(jsonViewObject) ? "" : JSONObject.toJSONString(jsonViewObject));
            opLog.setOperationTime(lastTime);
            opLog.setStartTime(new Date());
            opLog.setEndTime(new Date());
            opLog.setStatus(Objects.isNull(jsonViewObject) ? Constants.JsonView.STATUS_FAIL : jsonViewObject.getStatus());
            try {
                //用户客户端操作系统
                HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
                if (Objects.nonNull(request)) {
                    UserAgent userAgent = UserAgent.parseUserAgentString(request.getHeader("User-Agent"));
                    //获取客户端操作系统
                    OperatingSystem agentOs = userAgent.getOperatingSystem();
                    //获取客户端浏览器
                    String agentBrowser = "";
                    if (null != userAgent.getBrowser() && null != userAgent.getBrowserVersion()) {
                        agentBrowser = userAgent.getBrowser().getName() + " " + userAgent.getBrowserVersion().getVersion();
                    }
                    opLog.setOperationOS(agentOs.getName());
                    opLog.setOperationBrowser(agentBrowser);
                    opLog.setClientParam(request.getHeader("User-Agent"));

                    String agentIp = request.getHeader("X-Real-IP");
                    if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                        agentIp = request.getHeader("X-Forwarded-For");
                    }
                    if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                        agentIp = request.getHeader("Proxy-Client-IP");
                    }
                    if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                        agentIp = request.getHeader("WL-Proxy-Client-IP");
                    }
                    if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                        agentIp = request.getRemoteAddr();
                    }
                    opLog.setOperationIp("0:0:0:0:0:0:0:1".equals(agentIp) ? "127.0.0.1" : agentIp);
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            MqUtil.sendLogMessage(OPERATION_LOG.toString(), OPERATION_LOG.toString(), TrackableEntity.bean2Map(opLog));
        } catch (Exception e) {
            log.error("SendOperationLog error", e);
        }
    }

    /**
     * 抽象方法需要实现，得到基础服务接口
     *
     * @return
     */
    public abstract BaseInterfaceMapper<Entity> getBaseInterfaceMapper();
}
