package net.wanji.common.framework.dubbointerface.impl;

import net.wanji.common.framework.dubbointerface.BaseLogInterface;
import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.framework.rest.Page;
import net.wanji.common.tool.estool.EsConfiguration;
import net.wanji.common.tool.estool.EsUtil;
import net.wanji.common.utils.tool.IDGenerator;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.sort.SortBuilder;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;

import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.concurrent.ArrayBlockingQueue;

/**
 * @author wanji
 */
@Slf4j
public abstract class BaseLogInterfaceImpl implements BaseLogInterface {

    public static final String PAGE_NUM = "pageNum";
    public static final String TYPE = "type";
    public static final String PAGE_SIZE = "pageSize";
    public static final String CREATE_TIME = "createTime";
    public static final String START_TIME = "startTime";
    public static final String END_TIME = "endTime";
    public static final String API_ID = "apiId";
    public static final String API_ID_KEYWORD = "apiId.keyword";
    public static final String STATUS = "status";
    public static final String STATUS_KEYWORD = "status.keyword";

    public static EsUtil esUtil = EsUtil.getInstance();

    @Override
    public JsonViewObject logQuery(Map<String, Object> queryCondition) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        Page page = Page.newInstance();
        BoolQueryBuilder qb = QueryBuilders.boolQuery();

        if (queryCondition == null || queryCondition.isEmpty()) {
            return jsonViewObject.success(page, "查询成功");
        }
        try {
            String type = queryCondition.getOrDefault(TYPE, "").toString();
            if (StringUtils.isBlank(type)) {
                log.error("参数错误 type 为空");
                return jsonViewObject.fail("参数错误");
            }
            // 分页
            Page pageNumAndSize = getPageNumAndSize(queryCondition);
            int startRow = pageNumAndSize.getPageSize() * (pageNumAndSize.getPageNum() - 1);
            page.setPageNum(pageNumAndSize.getPageNum());
            page.setPageSize(pageNumAndSize.getPageSize());

            qb.must(QueryBuilders.rangeQuery(CREATE_TIME).from(queryCondition.get(START_TIME))
                    .to(queryCondition.get(END_TIME)));

            this.makeQueryBuilder(queryCondition, qb);

            // 排序
            Queue<SortBuilder> queue = getSortQueue(CREATE_TIME, SortOrder.DESC);

            page.setTotal(esUtil.searchDatasCount(EsConfiguration.getTransportClient(), type, type, qb).intValue());
            page.setRows(esUtil.searchDatas(EsConfiguration.getTransportClient(), type, type, qb, startRow, pageNumAndSize.getPageSize(), queue));
            jsonViewObject.success(page, "查询成功");
        } catch (Exception e) {
            log.error("logQuery error", e);
            jsonViewObject.fail(false, "查询失败");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject queryData(Map<String, Object> queryCondition) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        BoolQueryBuilder qb = QueryBuilders.boolQuery();

        try {
            String type = queryCondition.getOrDefault(TYPE, "").toString();
            if (StringUtils.isBlank(type)) {
                return jsonViewObject.fail("参数错误");
            }

            qb.must(QueryBuilders.rangeQuery(CREATE_TIME).from(queryCondition.get(START_TIME))
                    .to(queryCondition.get(END_TIME)));

            this.makeQueryBuilder(queryCondition, qb);

            // 排序
            Queue<SortBuilder> queue = getSortQueue(CREATE_TIME, SortOrder.DESC);

            List<Map<String, Object>> result = esUtil.searchDatas(EsConfiguration.getTransportClient(), type, type, qb, queue);
            jsonViewObject.success(result, "查询成功");
        } catch (Exception e) {
            log.error("queryData error", e);
            jsonViewObject.fail(false, "查询失败");
        }
        return jsonViewObject;
    }

    @Override
    public boolean saveServiceLog(String indexName, String typeName, Map<String, Object> logData) throws DubboProviderException {
        return esUtil.putIndexData(EsConfiguration.getTransportClient(), indexName, typeName, IDGenerator.getID(), logData);
    }

    // 分页
    public Page getPageNumAndSize(Map<String, Object> queryCondition) {
        Page pageNumAndSize = Page.newInstance();
        String s = queryCondition.getOrDefault(PAGE_NUM, 1).toString();
        int pageNum = 1;
        if (StringUtils.isNotBlank(s)) {
            pageNum = Integer.valueOf(s);
        }
        pageNumAndSize.setPageNum(pageNum);
        s = queryCondition.getOrDefault(PAGE_SIZE, 10).toString();
        int pageSize = 10;
        if (StringUtils.isNotBlank(s)) {
            pageSize = Integer.valueOf(s);
        }
        pageNumAndSize.setPageSize(pageSize);
        return pageNumAndSize;
    }

    // 排序
    public Queue<SortBuilder> getSortQueue(String sort, SortOrder type) {
        SortBuilder createTime = SortBuilders.fieldSort(sort).order(type);
        Queue<SortBuilder> queue = new ArrayBlockingQueue<>(2);
        queue.add(createTime);
        return queue;
    }



}
