package net.wanji.common.framework.exceptionhandler;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;


@RestControllerAdvice
@Slf4j
public class GlobalExceptionHandler {

    /**
     * IO异常处理
     */
    @ExceptionHandler(value = IOException.class)
    public JsonViewObject exceptionHandler(HttpServletRequest req, IOException e) {
        Throwable cause = e.getCause();
        String errorMessage;
        if (cause != null) {
            errorMessage = cause.getMessage();
        } else {
            StackTraceElement element = e.getStackTrace()[0];
            errorMessage = element.getClassName() + " " + element.getLineNumber();
        }
        log.error("发生IO异常:", e);
        return JsonViewObject.newInstance().fail("发生IO异常！位置在：" + errorMessage);
    }

    /**
     * 类类型转换异常处理
     */
    @ExceptionHandler(value = ClassCastException.class)
    public JsonViewObject exceptionHandler(HttpServletRequest req, ClassCastException e) {
        Throwable cause = e.getCause();
        String errorMessage;
        if (cause != null) {
            errorMessage = cause.getMessage();
        } else {
            StackTraceElement element = e.getStackTrace()[0];
            errorMessage = element.getClassName() + " " + element.getLineNumber();
        }
        log.error("发生类类型转换异常:", e);
        return JsonViewObject.newInstance().fail("发生类类型转换异常！位置在：" + errorMessage);
    }

    /**
     * 下表越界异常处理
     */
    @ExceptionHandler(value = IndexOutOfBoundsException.class)
    public JsonViewObject exceptionHandler(HttpServletRequest req, IndexOutOfBoundsException e) {
        Throwable cause = e.getCause();
        String errorMessage;
        if (cause != null) {
            errorMessage = cause.getMessage();
        } else {
            StackTraceElement element = e.getStackTrace()[0];
            errorMessage = element.getClassName() + " " + element.getLineNumber();
        }
        log.error("发生下标越界异常:", e);
        return JsonViewObject.newInstance().fail("发生下标越界异常！位置在：" + errorMessage);
    }

    /**
     * 处理空指针的异常
     */
    @ExceptionHandler(value = NullPointerException.class)
    public JsonViewObject exceptionHandler(HttpServletRequest req, NullPointerException e) {
        Throwable cause = e.getCause();
        String errorMessage;
        if (cause != null) {
            errorMessage = cause.getMessage();
        } else {
            StackTraceElement element = e.getStackTrace()[0];
            errorMessage = element.getClassName() + " " + element.getLineNumber();
        }
        log.error("发生空指针异常:", e);
        return JsonViewObject.newInstance().fail("发生空指针异常！位置在：" + errorMessage);
    }

    /**
     * 参数异常
     */
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public JsonViewObject exceptionHandler(HttpServletRequest req, MethodArgumentNotValidException e) {
        FieldError fieldError = e.getBindingResult().getFieldError();
        String errorMessage = "服务器内部错误";
        if (fieldError != null) {
            errorMessage = fieldError.getDefaultMessage();
        }
        log.error("发生参数异常:", e);
        return JsonViewObject.newInstance().fail("发生参数异常！原因是：" + errorMessage);
    }

    /**
     * 处理其他异常
     */
    @ExceptionHandler(value = Exception.class)
    public JsonViewObject exceptionHandler(HttpServletRequest req, Exception e) {
        Throwable cause = e.getCause();
        String errorMessage;
        if (cause != null) {
            errorMessage = cause.getMessage();
        } else {
            errorMessage = e.getMessage();
        }
        log.error("未知异常:", e);
        return JsonViewObject.newInstance().fail("未知异常！原因是：" + errorMessage);
    }
}
