package net.wanji.common.framework.i18n;

import org.apache.commons.lang3.StringUtils;
import java.lang.reflect.Field;

/**
 * @Description: 国际化词条自动处理工具类
 * @Author wanji
 * @Date 2019/11/30 11:20
 **/
public final class I18nResourceBundle {

    /**
     * @Description: 当前国际化语言环境，默认中文，外部可以动态修改，直接通过赋值即可更新语言环境，
     *               例：I18nResourceBundle.language = "en"
     *               修改后在通过I18nResourceBundle.getConstants("key")获取的值即可动态改变
     * @return
     * @Author wanji
     * @Date 2019/11/30 11:20
     **/
    public static volatile String language = "zh";


    /**
     * @Description: 当前支持的国际化语言，后续补充直接扩充枚举常量即可
     * @Author wanji
     * @Date 2019/11/30 11:21
     **/
    public enum LANGUAGE {
        /*中文简体，默认值*/
        ZH("zh"),
        /*英文*/
        EN("en");

        LANGUAGE(String zh) {
        }
    }

    /**
     * @Description: 工具类私有化构造函数，安全性考虑，禁止通过外部实例化
     * @Author wanji
     * @Date 2019/11/30 11:22
     **/
    private I18nResourceBundle() {
    }

    /**
     * @Description: 根据词条在Constants中定义的成员变量名称，获取成员变量值
     * 实现原理：通过Java反射，动态获取Constants中定义的成员变量值，
     * 结合当前语言环境，判断从Constants（默认中文）或者Constants_en（英文）中加载对应值
     * 后期扩展新语言，需要定义新Constants，并扩展LANGUAGE。
     *
     * @param key 词条键值
     * @return java.lang.String 返回字符串，当前仅支持字符串定义
     * @Author wanji
     * @Date 2019/11/29 22:12
     **/
    public static String getConstants(String key) {
        String constant = "";
        if (StringUtils.isEmpty(key)) {
            return constant;
        }
        try {
            Field filed;
            if (StringUtils.equalsIgnoreCase(LANGUAGE.EN.name(), language)) {
                filed = Constants_en.class.getDeclaredField(key);
            } else {
                filed = Constants.class.getDeclaredField(key);
            }
            //减少安全性检查，提高反射访问速度。Todo 性能对比
            filed.setAccessible(false);
            constant = String.valueOf(filed.get(key));
        } catch (Exception e) {
            return constant;
        }
        return constant;
    }
    /*
     * Todo 测试 性能对比，直接访问成员变量 vs 通过反射获取 vs Spring boot自带resourceBundle
     * Todo 测试 线程安全
     */
    /**
     * @Description:
     *
     * @param args Todo 测试 性能对比，直接访问成员变量 vs 通过反射获取 vs Spring boot自带resourceBundle
     *        Todo 测试 线程安全
     * @return void
     * @Author wanji
     * @Date 2019/11/30 13:43
     **/
    public static void main(String[] args) {
        I18nResourceBundle.language = "en";
        System.out.println(I18nResourceBundle.getConstants("SAVE_SUCCESS_MSG"));
        I18nResourceBundle.language = "zh";
        System.out.println(I18nResourceBundle.getConstants("SAVE_SUCCESS_MSG"));
    }
}
