package net.wanji.common.framework.rest;

import net.wanji.common.framework.domain.TrackableEntity;
import io.swagger.annotations.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;

/**
 * @author wanji
 * 修改：新增请求参数校验内容
 */
@Api(value = "RESTful服务基础接口", description = "RESTful服务基础接口")
@Validated
public interface AbstractRestServer<Entity extends TrackableEntity> {

    /**
     * 获取所有记录
     *
     * @return JsonViewObject
     */
    @ApiOperation(value = "获取所有记录", notes = "获取所有记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/byAll", produces = MediaType.APPLICATION_JSON)
    JsonViewObject getAll();

    /**
     * 根据条件分页查询记录
     *
     * @param page
     * @return JsonViewObject
     */
    @ApiOperation(value = "根据条件分页查询记录", notes = "根据条件分页查询记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/byPage", produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ResponseBody
    JsonViewObject getPage(@ApiParam(value = "查询条件和分页参数", required = true, example = "{\"pageSize\": 10, \"pageNum\": 1}") @RequestBody @Validated Page<Entity> page);

    /**
     * 根据条件查询记录
     *
     * @param entity 查询条件
     * @return JsonViewObject
     */
    @ApiOperation(value = "根据条件查询记录", notes = "根据条件查询记录", response = JsonViewObject.class, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/byCondition", produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    JsonViewObject getByWhere(@ApiParam(value = "查询条件", required = true) @RequestBody @Validated({ValidationGroups.Query.class}) Entity entity);

    /**
     * 根据id查询记录
     *
     * @param id
     * @return JsonViewObject
     */
    @ApiOperation(value = "根据id查询记录", notes = "根据id查询记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON)
    JsonViewObject getById(@ApiParam(value = "记录的id", required = true, example = "1") @PathVariable("id") @NotBlank(message = "查询id不能为空") String id);

    /**
     * 根据id删除
     *
     * @param ids
     * @return JsonViewObject
     */
    @ApiOperation(value = "根据多个id删除记录", notes = "根据多个id删除记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "ids", dataType = "String", required = true, value = "多个记录id，用逗号分隔", example = "1,2")
    })
    @GetMapping(value = "/deleting", produces = MediaType.APPLICATION_JSON)
    JsonViewObject deleteByIds(@QueryParam("ids") @NotBlank(message = "删除ids不能为空") String ids);

    /**
     * 新建记录
     *
     * @param entity
     * @return JsonViewObject
     */
    @ApiOperation(value = "新建记录", notes = "新建记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/creating", produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    JsonViewObject save(@ApiParam(value = "记录的JSON格式字符串", required = true) @RequestBody @Validated({ValidationGroups.Create.class}) Entity entity);

    /**
     * 修改记录
     *
     * @param entity
     * @return
     */
    @ApiOperation(value = "修改记录", notes = "修改记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/updating", produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    JsonViewObject update(@ApiParam(value = "记录的JSON格式字符串", required = true) @RequestBody @Validated({ValidationGroups.Update.class}) Entity entity);


    /**
     * 发送日志
     *
     * @param classNameCn 操作对象中文名
     * @param className 操作对象英文名
     * @param method 操作方法名
     * @param username 操作人
     * @param dbId 数据库id
     * @param jsonViewObject 返回对象
     * @param lastTime 操作时长
     */
   void sendOperationLog(String classNameCn,String className,String method,String username,String dbId,long lastTime,JsonViewObject jsonViewObject) ;


    }
