package net.wanji.common.framework.rest.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import net.wanji.common.framework.Constants;
import net.wanji.common.framework.domain.OperationLog;
import net.wanji.common.framework.domain.TrackableEntity;
import net.wanji.common.framework.dubbointerface.BaseDubboInterface;
import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.common.framework.i18n.I18nResourceBundle;
import net.wanji.common.framework.rest.AbstractRestServer;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.framework.rest.Page;
import com.google.common.collect.Maps;
import net.wanji.common.tool.mqtool.MqUtil;
import eu.bitwalker.useragentutils.OperatingSystem;
import eu.bitwalker.useragentutils.UserAgent;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.core.Context;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static net.wanji.common.framework.Constants.LogType.OPERATION_LOG;

/**
 * @author wanji
 */
//@RequiresAuthentication
@Slf4j
public abstract class AbstractRestServerImpl<Entity extends TrackableEntity> implements AbstractRestServer<Entity> {

    @Context
    protected HttpServletResponse response;

    @Context
    protected HttpServletRequest request;


    /**
     * 抽象方法，子类需要实现，得到基础服务接口
     *
     * @return 各子类相对应的Provider接口
     */
    public abstract BaseDubboInterface<Entity> getBaseDubboInterface();

    /**
     * {
     * "content": "{"endRowNum":10,"objCondition":null,"pageNumber":1,"pageSize":10,"startRowNum":0,"total":2,"totalPageNum":1,
     * "rows":[
     * {"createDate":"2015-09-29 15:30:42","groupName":"test1","id":"C8879FFFB2064E66BB7A7D4ED052C9DE","username":null,"webPort":"5052"},
     * {"createDate":"2015-09-29 14:51:46","groupName":"test2","id":"122830C1DC334517BC7BCE79F62D4FD5","username":null,"webPort":"5052"}
     * ]}",
     * "message": "",
     * "status": "success"
     * }
     *
     * @param page
     * @return
     */
    @Override
    public JsonViewObject getPage(@RequestBody Page<Entity> page) {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        String jsonStr = JSON.toJSONString(page);
        try {
            Map<String, Object> mapBean = Maps.newHashMap();
            if (page != null) {
                if (page.getCondition() != null && page.getCondition() instanceof Map) {
                    mapBean = (Map<String, Object>) page.getCondition();
                }
            }
            page = this.getBaseDubboInterface().findByPage(page, mapBean);
            jsonView.success(page);
        } catch (DubboProviderException e) {
            jsonView.fail(e);
            log.error("{} getPage error, jsonStr:{}", this.getClass().getSimpleName(), jsonStr, e);
        }finally {
            //this.sendOperationLog("根据条件分页查询记录->"+jsonStr,this.getClass().getSimpleName(),"resetPwd","","", System.currentTimeMillis()-start,jsonView);
        }
        return jsonView;
    }

    /**
     * {
     * "content": "[
     * {"createDate":"2015-09-29 15:30:42","id":"C8879FFFB2064E66BB7A7D4ED052C9DE","name":"test1","status":"0"},
     * {"createDate":"2015-09-29 14:51:46","id":"122830C1DC334517BC7BCE79F62D4FD5","name":"test2","status":"0"}
     * ]",
     * "message": "",
     * "status": "success"
     * }
     *
     * @return
     */
    @Override
    public JsonViewObject getAll() {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        try {
            List<Entity> list = this.getBaseDubboInterface().findAll();
            jsonView.success(list);
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }finally {
            //this.sendOperationLog("获取所有记录->",this.getClass().getSimpleName(),"resetPwd","","", System.currentTimeMillis()-start,jsonView);
        }
        return jsonView;
    }

    /**
     * {
     * "content": "[
     * {"createDate":"2015-09-29 15:30:42","id":"C8879FFFB2064E66BB7A7D4ED052C9DE","name":"test1","status":"0"},
     * {"createDate":"2015-09-29 14:51:46","id":"122830C1DC334517BC7BCE79F62D4FD5","name":"test2","status":"0"}
     * ]",
     * "message": "",
     * "status": "success"
     * }
     *
     * @param entity
     * @return
     */
    @Override
    public JsonViewObject getByWhere(@RequestBody Entity entity) {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        String jsonStr = JSON.toJSONString(entity);
        try {
            //参数校验过程中修改，兼容实现，restful入参不用map，便于进行参数逐个校验
            Map params = JSONObject.parseObject(JSONObject.toJSONString(entity), Map.class);
            List list = this.getBaseDubboInterface().findByMap(params);
            jsonView.success(list);
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getByWhere error，jsonStr:{}", this.getClass().getSimpleName(), jsonStr, e);
        }finally {
            //this.sendOperationLog("根据条件查询记录->"+jsonStr,this.getClass().getSimpleName(),"resetPwd","","", System.currentTimeMillis()-start,jsonView);
        }
        return jsonView;
    }

    /**
     * {
     * "content": "{"groupId":"D54FAB067D6F47A99136210ED640368E","id":"C8879FFFB2064E66BB7A7D4ED052C9DE","status":"0","webPort":"5052"}",
     * "message": "",
     * "status": "success"
     * }
     *
     * @param id
     * @return JsonViewObject
     */
    @Override
    public JsonViewObject getById(@PathVariable String id) {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        try {
            Entity entity = this.getBaseDubboInterface().findById(id);
            jsonView.success(entity);
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("AbstractRestServerImpl getById error, id:{}", id, e);
        }finally {
            //this.sendOperationLog("根据id查询记录->"+id,this.getClass().getSimpleName(),"resetPwd","","", System.currentTimeMillis()-start,jsonView);
        }
        return jsonView;
    }

    @Override
    public JsonViewObject deleteByIds(String ids) {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        String[] idArray = ids.split(",");
        try {
            if (idArray.length > 0) {
                for (String id : idArray) {
                    jsonView = this.getBaseDubboInterface().deleteById(id);
                }
            }
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("DELETE_FAILED_MSG"));
            log.error("AbstractRestServerImpl deleteByIds error, id:{}", ids, e);
        }finally {
            this.sendOperationLog("根据多个id删除记录->"+ids,this.getClass().getSimpleName(),"resetPwd","","", System.currentTimeMillis()-start,jsonView);
        }
        return jsonView;
    }

    @Override
    public JsonViewObject save(@RequestBody Entity entity) {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        try {
            if (entity != null) {
                jsonView = this.getBaseDubboInterface().save(entity);
            }
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("SAVE_FAILED_MSG"));
            log.error("AbstractRestServerImpl save error, jsonStr:{}", JSON.toJSONString(entity), e);
        }finally {
            this.sendOperationLog("新增记录->"+entity,this.getClass().getSimpleName(),"resetPwd","","", System.currentTimeMillis()-start,jsonView);
        }
        return jsonView;
    }

    @Override
    public JsonViewObject update(@RequestBody Entity entity) {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();
        try {
            if (entity != null) {
                jsonView = this.getBaseDubboInterface().update(entity);
            }
        } catch (DubboProviderException e) {
            jsonView.fail(I18nResourceBundle.getConstants("UPDATE_FAILED_MSG"));
            log.error("AbstractRestServerImpl update error, jsonStr:{}", JSON.toJSONString(entity), e);
        }finally {
            this.sendOperationLog("修改记录->"+entity,this.getClass().getSimpleName(),"resetPwd","","", System.currentTimeMillis()-start,jsonView);
        }
        return jsonView;
    }


    @Override
    public void sendOperationLog(String classNameCn,String className,String method,String username,String dbId,long lastTime,JsonViewObject jsonViewObject) {
        OperationLog opLog = new OperationLog();
        try {
            opLog.setResName(classNameCn);
            opLog.setResType(className);
            opLog.setOperationType(method);
            opLog.setOperationUser(username);
            opLog.setDbId(dbId);
            opLog.setDes(Objects.isNull(jsonViewObject)?"":JSONObject.toJSONString(jsonViewObject));
            opLog.setOperationTime(lastTime);
            opLog.setStartTime(new Date());
            opLog.setEndTime(new Date());
            opLog.setStatus(Objects.isNull(jsonViewObject)? Constants.JsonView.STATUS_FAIL:jsonViewObject.getStatus());
            try {
                //用户客户端操作系统
                if (request == null) {
                    request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
                }
                UserAgent userAgent = UserAgent.parseUserAgentString(request.getHeader("User-Agent"));
                //获取客户端操作系统
                OperatingSystem agentOs = userAgent.getOperatingSystem();
                //获取客户端浏览器
                String agentBrowser = "";
                if (null != userAgent.getBrowser() && null != userAgent.getBrowserVersion()) {
                    agentBrowser = userAgent.getBrowser().getName() + " " + userAgent.getBrowserVersion().getVersion();
                }
                opLog.setOperationOS(agentOs.getName());
                opLog.setOperationBrowser(agentBrowser);
                opLog.setClientParam(request.getHeader("User-Agent"));

                String agentIp = request.getHeader("X-Real-IP");
                if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                    agentIp = request.getHeader("X-Forwarded-For");
                }
                if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                    agentIp = request.getHeader("Proxy-Client-IP");
                }
                if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                    agentIp = request.getHeader("WL-Proxy-Client-IP");
                }
                if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                    agentIp = request.getRemoteAddr();
                }
                opLog.setOperationIp("0:0:0:0:0:0:0:1".equals(agentIp) ? "127.0.0.1" : agentIp);
            }catch (Exception ex){
                ex.printStackTrace();
            }
            MqUtil.sendLogMessage(OPERATION_LOG.toString(), OPERATION_LOG.toString(), TrackableEntity.bean2Map(opLog));
        } catch (Exception e) {
            log.error("SendOperationLog error", e);
        }
    }
}
