package net.wanji.common.utils.cache;

import net.wanji.common.framework.spring.ServiceBeanContext;
import net.wanji.common.utils.tool.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.ignite.Ignition;
import org.apache.ignite.cache.CacheAtomicityMode;
import org.apache.ignite.cache.CacheMode;
import org.apache.ignite.cache.PartitionLossPolicy;
import org.apache.ignite.cache.query.ScanQuery;
import org.apache.ignite.client.ClientCache;
import org.apache.ignite.client.ClientCacheConfiguration;
import org.apache.ignite.client.ClientException;
import org.apache.ignite.client.IgniteClient;
import org.apache.ignite.configuration.ClientConfiguration;

import javax.cache.expiry.CreatedExpiryPolicy;
import javax.cache.expiry.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import static net.wanji.common.framework.Constants.igniteCacheName.BP_DESIGN_CACHE;

/**
 * @ClassName IgniteClientCacheUtil
 * @Description ignite 缓存 工具类
 * @Author wanji
 * @Date 2020/7/9 15:15
 * @Version 1.0
 */
@Slf4j
public class IgniteClientCacheUtil {

    private static String addresses = ServiceBeanContext.getContext().getEnvironment().getProperty("ignite-client.address");
    //private static String addresses = "121.36.56.157:10800";

    private static IgniteClient igniteClient = null;

    static {
        IgniteClientCacheUtil.initIgniteClient();
    }

    /**
     * 初始化缓存管理容器
     */
    private static void initIgniteClient() {
        try {
            if (igniteClient == null) {
                igniteClient = Ignition.startClient(getClientConfiguration());
                log.info("IgniteClientCacheUtil initIgniteClient success");
            }
        } catch (Exception e) {
            log.error("IgniteClientCacheUtil initIgniteClient error.", e);
        }
    }

    /**
     * 初始化缓存
     */
    public static ClientCache<String, Object> initCache(String cacheName) throws Exception {
        log.info("IgniteClientCacheUtil initCache , cacheName : {}", cacheName);
        return IgniteClientCacheUtil.getCache(cacheName);
    }

    /**
     * 初始化缓存
     */
    public static ClientCache<String, Object> initCache(String cacheName, Long expireTime) throws Exception {
        log.info("IgniteClientCacheUtil initCache , cacheName : {} ,expire :{} ", cacheName, expireTime);
        return IgniteClientCacheUtil.getCache(cacheName, expireTime);
    }

    private static ClientConfiguration getClientConfiguration() {
        log.info("IgniteClientCacheUtil getClientConfiguration , addresses : {}", addresses);
        ClientConfiguration configuration = new ClientConfiguration();
        configuration.setAddresses(addresses);
        return configuration;
    }

    /**
     * 检测 igniteClient
     */
    private static void checkIgniteClient() {
        if (null == igniteClient) {
            throw new IllegalArgumentException("调用前请先初始化 IgniteClient 值：IgniteClientCacheUtil.checkIgniteClient");
        }
    }

    /**
     * 向指定容器中设置值
     *
     * @param cacheName 容器名
     * @param key       键
     * @param value     值
     * @return 返回真
     * @throws Exception 异常
     */
    public static boolean setValue(String cacheName, String key, Object value) throws Exception {
        checkIgniteClient();
        try {
            log.info("IgniteClientCacheUtil setValue , cacheName : {} , key : {} , value : {}", cacheName, key, value);
            ClientCache<String, Object> cache = getCache(cacheName);
            if (StringUtils.isBlank(key)) {
                throw new Exception("IgniteClientCacheUtil setValue , key == null");
            }
            cache.put(key, value);
            return true;
        } catch (Exception e) {
            log.error("IgniteClientCacheUtil setValue error, cacheName = {}, key = {}, value = {}.", cacheName, key, value, e);
            throw new Exception("set cache " + cacheName + " failed!!!");
        }
    }

    /**
     * 从ehcache的指定容器中取值
     *
     * @param key 键
     * @return 返回Object类型的值
     */
    public static Object getValue(String cacheName, String key) throws Exception {
        checkIgniteClient();
        Object object = null;
        try {
            ClientCache<String, Object> cache = getCache(cacheName);
            if (StringUtils.isNotBlank(key)) {
                object = cache.get(key);
            }
        } catch (Exception e) {
            log.error("IgniteClientCacheUtil getValue error, cacheName = {}, key = {}.", cacheName, key, e);
            throw new Exception("IgniteClientCacheUtil getValue error, cacheName = " + cacheName + ", key = " + key, e);
        }
        return object;
    }

    /**
     * 获取Cache所有的Keys
     *
     * @param cacheName
     * @return
     */
    public static List<String> getKeys(String cacheName) throws Exception {
        log.info("IgniteClientCacheUtil getKeys , cacheName : {}", cacheName);
        return getKeys(getCache(cacheName));
    }

    /**
     * 获取Keys下的所有值
     *
     * @param cacheName
     * @param keys
     * @return
     * @throws Exception
     */
    public static List<Object> getValues(String cacheName, List<String> keys) throws Exception {
        log.info("IgniteClientCacheUtil getValues ,keys :{} ", keys.toArray());
        checkIgniteClient();
        List<Object> values = new ArrayList<>();
        keys.forEach(key -> {
            try {
                ClientCache<String, Object> cache = getCache(cacheName);
                if (StringUtils.isNotBlank(key)) {
                    values.add(cache.get(key));
                }
            } catch (Exception e) {
                log.error("IgniteClientCacheUtil getValue error, cacheName = {}, key = {}.", cacheName, key, e);
            }
        });
        return values;

    }

    private static List<String> getKeys(ClientCache<String, Object> cache) {
        checkIgniteClient();
        List<String> keys = new ArrayList<>();
        cache.query(new ScanQuery<>(null)).forEach(entry -> keys.add((String) entry.getKey()));
        return keys;
    }

    private static ClientCache<String, Object> getCache(String cacheName) throws Exception {
        checkIgniteClient();
        if (StringUtils.isBlank(cacheName)) {
            throw new Exception("IgniteClientCacheUtil getCache , cacheName == null");
        }
        ClientCacheConfiguration configuration = new ClientCacheConfiguration();
        configuration.setName(cacheName);
        configuration.setCacheMode(CacheMode.REPLICATED);
        configuration.setPartitionLossPolicy(PartitionLossPolicy.READ_ONLY_SAFE);
        configuration.setAtomicityMode(CacheAtomicityMode.TRANSACTIONAL);
        return igniteClient.getOrCreateCache(configuration);
    }

    private static ClientCache<String, Object> getCache(String cacheName, Long expireTime) throws Exception {
        checkIgniteClient();
        if (StringUtils.isBlank(cacheName)) {
            throw new Exception("IgniteClientCacheUtil getCache , cacheName == null");
        }
        ClientCacheConfiguration configuration = new ClientCacheConfiguration();
        configuration.setName(cacheName);
        configuration.setCacheMode(CacheMode.REPLICATED);
        configuration.setExpiryPolicy(CreatedExpiryPolicy.factoryOf(new Duration(TimeUnit.SECONDS, expireTime)).create());
        configuration.setPartitionLossPolicy(PartitionLossPolicy.READ_ONLY_SAFE);
        configuration.setAtomicityMode(CacheAtomicityMode.TRANSACTIONAL);
        return igniteClient.getOrCreateCache(configuration);
    }

    /**
     * 删除 ignite client cache 中的 key
     *
     * @param cacheName 容器名
     * @param key       键
     * @return 真
     * @throws Exception 失败抛出异常
     */
    public static boolean removeKey(String cacheName, String key) throws Exception {
        checkIgniteClient();
        try {
            log.info("IgniteClientCacheUtil removeKey , cacheName : {} , key : {}", cacheName, key);
            ClientCache<String, Object> cache = getCache(cacheName);
            cache.remove(key);
            return true;
        } catch (Exception e) {
            log.error("IgniteClientCacheUtil removeKey error, cacheName = {}, key = {}.", cacheName, key, e);
            throw new Exception("remove cache " + cacheName + " failed!!!");
        }
    }

    /**
     * 删除 ignite client cache 中 包含 containsKey 的 key
     *
     * @param cacheName   容器名
     * @param containsKey 键
     * @return 真
     * @throws Exception 失败抛出异常
     */
    public static boolean removeContainsKey(String cacheName, String containsKey) throws Exception {
        checkIgniteClient();
        try {
            ClientCache<String, Object> cache = getCache(cacheName);
            List<String> keys = IgniteClientCacheUtil.getKeys(cache);
            for (String key : keys) {
                if (key.contains(containsKey)) {
                    cache.remove(key);
                    log.info("IgniteClientCacheUtil removeKey , cacheName : {} , key : {}", cacheName, key);
                }
            }
            return true;
        } catch (Exception e) {
            log.error("IgniteClientCacheUtil removeKey error, cacheName = {}, containsKey = {}.", cacheName, containsKey, e);
            throw new Exception("remove cache " + cacheName + " failed!!!");
        }
    }

    /**
     * 删除 ignite client cache 中所有的 key
     *
     * @param cacheName 容器名
     * @return 真
     * @throws Exception 失败抛出异常
     */
    public static boolean removeAllKey(String cacheName) throws Exception {
        checkIgniteClient();
        try {
            log.info("IgniteClientCacheUtil removeAllKey , cacheName : {}", cacheName);
            ClientCache<String, Object> cache = getCache(cacheName);
            cache.removeAll();
            return true;
        } catch (Exception e) {
            log.error("IgniteClientCacheUtil removeAllKey error, cacheName = {}.", cacheName, e);
            throw new Exception("remove cache " + cacheName + " failed!!!");
        }
    }

    /**
     * 移除cache
     *
     * @param cacheName
     */
    public static void removeCache(String cacheName) {
        checkIgniteClient();
        log.info("IgniteClientCacheUtil removeCache , cacheName : {}", cacheName);
        try {
            igniteClient.destroyCache(cacheName);
        } catch (ClientException e) {
            log.info("IgniteClientCacheUtil removeCache , cacheName : {} Cache does not exist ", cacheName);
        }
    }

    public static void main(String[] args) throws Exception {
        String cacheName = "VECTOR_TILE";

        IgniteClientCacheUtil.initCache(cacheName);

        IgniteClientCacheUtil.removeAllKey(cacheName);
       // removeKey(cacheName, "92460E87D880442E951A95026856590F");
        /*try {
            setValue(cacheName, "test", "test");
        } catch (Exception e) {
            log.error("setValue error", e);
        }
        try {
            System.out.println(getValue(cacheName, "test"));
        } catch (Exception e) {
            log.error("getValue error", e);
        }

        System.out.println(new Date().getTime()); // 1594290889200  1594290962412
        for (int i = 0; i < 10000; i++) {
            try {
                setValue(cacheName, String.valueOf(i), i);
            } catch (Exception e) {
                log.error("setValue error" + i, e);
            }
        }
        System.out.println(new Date().getTime()); // 1594290891526  1594290964803
        for (int i = 0; i < 10000; i++) {
            try {
                getValue(cacheName, String.valueOf(i));
//                Object value = getValue(cacheName, String.valueOf(i));
//                System.out.println(value);
            } catch (Exception e) {
                log.error("getValue error" + i, e);
            }
        }
        System.out.println(new Date().getTime()); // 1594290894087  1594290967215*/
    }

    /*
     * CacheMode 缓存操作模式
     *      PARTITIONED 分区模式:所有数据均匀分布在每个节点上,默认的模式
     *      REPLICATED 复制模式:所有数据复制到每个节点上,每一个节点都保存其他节点的备份
     *      LOCAL 本地模式
     *
     * PartitionLossPolicy 分区丢失策略
     *      READ_ONLY_SAFE：所有缓存/表的写操作都会抛出异常，在线分区的读操作是可以的，丢失分区的读操作会抛出异常；
            READ_ONLY_ALL：包括丢失分区在内的所有分区都是可以读的，所有分区的写操作都会抛出异常，从丢失的分区读取数据的结果是未定义的，而且不同的节点返回结果可能不同；
            READ_WRITE_SAFE：在线分区的读写都是可以的，丢失分区的读写操作都会抛出异常；
            READ_WRITE_ALL：所有的读写都会继续进行，就像所有分区都处于一致状态那样（就像没有分区丢失），从丢失分区的读操作是未定义的，并且不同节点返回结果可能不同；
            IGNORE：该模式不会标记丢失的分区为丢失状态，假定没有发生分区丢失，并且立即清除分区丢失状态。从技术上来说，分区不会被加入lostPartitions列表，这是与READ_WRITE_ALL模式的主要区别。从丢失的分区读取数据的结果是未定义的，而且不同的节点返回结果可能不同，IGNORE模式为默认模式。
     *
     * writeSynchronizationMode 写同步模式
     *      FULL_SYNC	客户端节点要等待所有相关远程节点的写入或者提交完成（主和备）。
            FULL_ASYNC	客户端不需要等待参与节点的响应。这时远程节点可能会在任何缓存写入方法完成或Transaction.commit()方法完成后稍微更新其状态。
            PRIMARY_SYNC	这是默认模式，客户端节点会等待主节点的写或者提交完成，但不会等待备份节点的更新完成。
     *
     * 主备副本 backups
     *      在PARTITIONED模式下，数据的主键归属的节点叫做这些主键的主节点，对于缓存的数据，也可以可选地配置任意多个备份节点。如果副本数量大于0，那么Ignite会自动地为每个独立的键赋予备份节点，比如，如果副本数量为1，那么数据网格内缓存的每个键都会有2个备份，一主一备。
     *
     * 原子化模式 CacheAtomicityMode
     *      TRANSACTIONAL和TRANSACTIONAL_SNAPSHOT模式支持完全兼容ACID的事务，TRANSACTIONAL模式只支持基于Java API的键-值事务，这个模式的事务有不同的并发模型和隔离级别。
     *      ATOMIC模式因为避免了事务锁，所以性能更好，但是仍然提供了单个数据的原子性和一致性。ATOMIC模式的另一个不同是批量写，比如putAll(...)和removeAll(...)方法不在一个事务中执行，因此可能部分失败并抛出CachePartialUpdateException，它里面包含了更新失败的键列表。
     * */
}
