package net.wanji.common.utils.file;


import com.sun.xfile.XFile;
import com.sun.xfile.XFileInputStream;
import com.sun.xfile.XFileOutputStream;
import lombok.extern.slf4j.Slf4j;

import java.io.*;

/**
 * 使用xfile访问nfs文件系统
 *
 * @author
 * @date 2019/2/17
 */
@Slf4j
public class XFileUtils {

    /**
     * 主机ip
     */
    private String ip;
    /**
     * 共享目录
     */
    private String share;

    public XFileUtils(String ip, String share) {
        this.ip = ip;
        this.share = share;
    }

    private String getBaseSharePath() {
        String path = (ip + "/" + share + "/").replace("\\\\", "/").replace("//", "/");
        return "nfs://" + path;
    }

    public String getRemoteDir(String relativePath) {
        String dirPath = "";
        String remoteRoot = this.getBaseSharePath();
        relativePath = relativePath.replace("\\\\", "/");
        if (relativePath.startsWith("/")) {
            dirPath = remoteRoot + relativePath.substring(1);
        } else {
            dirPath = remoteRoot + relativePath;
        }
        return dirPath;
    }

    /**
     * 在远程共享文件系统中创建文件夹（支持多级目录创建）
     *
     * @param filePath
     * @throws Exception
     */
    public boolean mkdirs(String filePath) throws Exception {
        boolean flag = true;
        try {
            XFile xFile = new XFile(getRemoteDir(filePath));
            if (!xFile.exists()) {
                xFile.mkdirs();
                log.error("创建成功！");
            }
        } catch (Exception e) {
            flag = false;
            log.error("创建目录失败！", e);
            throw e;
        }
        return flag;
    }
    /**
     * 判断文件或目录是否存在
     *
     * @param filePath
     * @return
     * @throws Exception
     */
    public boolean exists(String filePath) throws Exception {
        boolean flag = false;
        try {
            XFile xFile = new XFile(getRemoteDir(filePath));
            if (xFile.exists()) {
                flag = true;
            }
        } catch (Exception e) {
            flag = false;
            log.error("exists fails！", e);
            throw e;
        }
        return flag;
    }

    /**
     * 读取远程文件
     *
     * @param remoteFileName
     * @return
     * @throws Exception
     */
    public InputStream readFileToInputStream(String remoteFileName) throws Exception {
        try {
            XFile file = new XFile(remoteFileName);
            if (file.exists()) {
                return new XFileInputStream(file);
            } else {
                log.debug("文件" + remoteFileName + "不存在");
                return null;
            }
        } catch (Exception e) {
            throw new Exception("create xFile Object exception by remoteFileName(" + remoteFileName + ")", e);
        }
    }

    /**
     * 读取远程文件
     *
     * @param remoteFileName
     * @return
     * @throws Exception
     */
    public InputStream readFileToConvertInputStream(String remoteFileName) throws Exception {
        try {
            XFile file = new XFile(remoteFileName);
            if (file.exists()) {
                return new XFileInputStream(file);
            } else {
                log.debug("文件" + remoteFileName + "不存在");
                return null;
            }
        } catch (Exception e) {
            throw new Exception("create xFile Object exception by remoteFileName(" + remoteFileName + ")", e);
        }
    }



    /**
     * 将数据流写入到远程文件系统中
     *
     * @return
     * @throws Exception
     */
    public boolean writeInputStreamToFile(InputStream inputStream, String remotedir, String remoteFilename) throws Exception {
        boolean isWriteSuccess = false;
        String fileFullName = remotedir + "/" + remoteFilename;
        XFileOutputStream xFileOutputStream = null;
        try {
            XFile file = new XFile(fileFullName);
            if (file.exists()) {
                file.delete();
            } else {
                XFile parent = new XFile(file.getParent());
                if (!parent.exists()) {
                    parent.mkdirs();
                }
            }
            xFileOutputStream = new XFileOutputStream(file);
            int len = 0;
            byte[] buffer = new byte[1024];
            while ((len = inputStream.read(buffer)) != -1) {
                xFileOutputStream.write(buffer, 0, len);
            }
            isWriteSuccess = true;
        } catch (Exception e) {
            throw new Exception("create xFile exception,{fileFullName:" + fileFullName + "}," + e.getMessage(), e);
        } finally {
            if (inputStream != null) {
                inputStream.close();
            }
            if (xFileOutputStream != null) {
                xFileOutputStream.flush();
                xFileOutputStream.close();
            }
        }
        return isWriteSuccess;
    }


    /**
     * 删除文件夹
     *
     * @param folderPath
     * @throws Exception
     */
    public boolean delFolder(String folderPath) throws Exception {
        boolean flag = false;
        try {
            if (!folderPath.endsWith("/")) {
                folderPath = folderPath + "/";
            }
            XFile xFile = new XFile(folderPath);
            if (xFile.exists()) {
                if (xFile.isDirectory()) {
                    String[] paths = xFile.list();
                    for (String path : paths) {
                        String childPath = folderPath + path;
                        this.delFolder(childPath);
                    }
                }
                xFile.delete();
            }
            flag = true;
        } catch (Exception e) {
            throw new Exception("delete directory exception," + e.getMessage(), e);
        }
        return flag;
    }


    /**
     * 删除文件
     *
     * @param fullRemoteFileName
     * @return
     * @throws Exception
     */
    public boolean delRemoteFile(String fullRemoteFileName) throws Exception {
        boolean flag = false;
        try {
            XFile file = new XFile(fullRemoteFileName);
            if (file.exists()) {
                file.delete();
            }
            flag = true;
        } catch (Exception e) {
            throw new Exception("deleted remote exception by fileName(" + fullRemoteFileName + ")," + e.getMessage(), e);
        }
        return flag;
    }
    public boolean delFile(String fullRemoteFileName) throws Exception {
        boolean flag = false;
        try {
            XFile file = new XFile(fullRemoteFileName);
            flag = file.delete();
        } catch (Exception e) {
            throw new Exception("deleted remote exception by fileName(" + fullRemoteFileName + ")," + e.getMessage(), e);
        }
        return flag;
    }



    /**
     * 下载远程文件至本地
     *
     * @param localFilePath
     * @param remoteDir
     * @return
     * @throws Exception
     */
    public boolean downloadToLocal(String localFilePath, String remoteDir) throws Exception {
        boolean flag = true;
        try {
                XFile xFile = new XFile(remoteDir);
            if (!xFile.exists()) {
                flag = false;
                throw new Exception("remote file[" + remoteDir + "] does't exist");
            }
            XFileInputStream inputStream = new XFileInputStream(xFile);
            File file=new File(localFilePath);
            if(!file.exists()){
                file.getParentFile().mkdirs();
            }
            FileOutputStream outputStream = new FileOutputStream(file);
            int len = 0;
            byte[] buffer = new byte[1024];
            while ((len = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, len);
            }
            inputStream.close();
            outputStream.close();
        } catch (Exception e) {
            flag = false;
            throw new Exception("download remote(" + remoteDir + ") to local" + localFilePath + " path exception," + e.getMessage(), e);
        }
        return flag;
    }

    public boolean renameName(String oldName, String newName) {
        XFile xFile = new XFile(getRemoteDir(oldName));
        XFile newFile = new XFile(getRemoteDir(newName));
        boolean b = xFile.renameTo(newFile);
        return b;
    }

    /**
     * 传入文件路径 将文件转为字节
     *
     * @param filePath
     * @return
     * @throws Exception
     */
    public byte[] readFileToBytes(String filePath) throws Exception {
        byte[] bytes = null;
        try {
            XFile file = new XFile(filePath);
            if (file.exists()) {
                XFileInputStream xFileInputStream = new XFileInputStream(file);
                bytes = inputStreamToBytes(xFileInputStream);
            } else {
                log.debug("文件" + filePath + "不存在");
                return null;
            }
        } catch (Exception e) {
            throw new Exception("create xFile Object exception by remoteFileName(" + filePath + ")", e);
        }
        return bytes;
    }

    /**
     * 将流转为字节
     *
     * @param inStream
     * @return
     * @throws Exception
     */
    public static byte[] inputStreamToBytes(InputStream inStream) throws Exception {
        byte[] bytes = null;
        try {
            ByteArrayOutputStream swapStream = new ByteArrayOutputStream();
            byte[] buff = new byte[100];
            int rc = 0;
            while ((rc = inStream.read(buff, 0, 100)) > 0) {
                swapStream.write(buff, 0, rc);
            }
            bytes = swapStream.toByteArray();
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception(e.getMessage(), e);
        }
        return bytes;
    }

    /**
     * 将字节流数据写入到远程文件系统中
     *
     * @param bytes
     * @return
     * @throws Exception
     */
    public boolean writeByteArrayToFile(byte[] bytes, String remoteDir, String remoteName) throws Exception {
        InputStream in = byteToInputStream(bytes);
        return writeInputStreamToFile(in, remoteDir, remoteName);
    }

    /**
     * 字节转换InputStream
     *
     * @param in
     * @return
     */
    public static InputStream byteToInputStream(byte[] in) throws Exception {
        ByteArrayInputStream is = new ByteArrayInputStream(in);
        return is;
    }


    public static void main(String[] args) throws Exception {
        XFileUtils xFileUtils = new XFileUtils("10.10.1.86", "/data/share/business/actionA/");

        boolean b = xFileUtils.downloadToLocal("/opt/bussinsess/ActionA-1.0.0.jar", xFileUtils.getRemoteDir("ActionA-1.0.0.jar"));
        System.out.println(b);
    }
}
