package net.wanji.common.utils.tool;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.springframework.cglib.beans.BeanMap;

import java.lang.reflect.Field;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @ClassName BeanMapUtils
 * @Description 用于Map和domain类的相互转换，比fastjson效率高
 * @Author ${wanji}
 * @Date 2020/6/19 13:42
 * @Version 1.0
 */
public class BeanMapUtils {

    private static Pattern linePattern = Pattern.compile("_(\\w)");
    private static Pattern humpPattern = Pattern.compile("[A-Z]");
    /** 下划线转驼峰 */
    public static String lineToHump(String str) {
        str = str.toLowerCase();
        Matcher matcher = linePattern.matcher(str);
        StringBuffer sb = new StringBuffer();
        while (matcher.find()) {
            matcher.appendReplacement(sb, matcher.group(1).toUpperCase());
        }
        matcher.appendTail(sb);
        return sb.toString();
    }

    /** 驼峰转下划线 */
    public static String humpToLine(String str) {
        Matcher matcher = humpPattern.matcher(str);
        StringBuffer sb = new StringBuffer();
        while (matcher.find()) {
            matcher.appendReplacement(sb, "_" + matcher.group(0).toLowerCase());
        }
        matcher.appendTail(sb);
        return sb.toString();
    }

    /**
     * 将对象装换为map
     * @param bean
     * @return
     */
    public static <T> Map<String, Object> beanToMap(T bean) {
        Map<String, Object> map = Maps.newHashMap();
        if (bean != null) {
            BeanMap beanMap = BeanMap.create(bean);
            for (Object key : beanMap.keySet()) {
                map.put(key+"", beanMap.get(key));
            }
        }
        return map;
    }

    /**
     * 将map装换为javabean对象 TODO bug 待处理，将map对象转换为bean时，如果属性中首字母为大写会转换不过去，待处理
     * @param map
     * @param clazz
     * @return
     */
    /*public static <T> T mapToBean(Map<String, Object> map,T bean) {
        BeanMap beanMap = BeanMap.create(bean);
        beanMap.putAll(map);
        return bean;
    }*/

    public static <T> T mapToBean(Map<String, Object> map,Class<T> clazz) throws Exception {
        T bean = null;
        try {
            bean = clazz.newInstance();
            for (Field field : getAllFields(clazz)) {
                if (map.containsKey(humpToLine(field.getName()))) {
                    boolean flag = field.isAccessible();
                    field.setAccessible(true);
                    Object object = map.get(humpToLine(field.getName()));
                    if (object != null) {
                        if (field.getType().isAssignableFrom(object.getClass())) {
                            field.set(bean, object);
                        } else {
                            //判断属性类型 进行转换,map中存放的是Object对象需要转换 实体类中有多少类型就加多少类型,实体类属性用包装类;
                            if (field.getType().toString().contains("Long")) {
                                field.set(bean, Long.parseLong(object.toString()));
                            }
                            if (field.getType().toString().contains("String")) {
                                field.set(bean, object.toString());
                            }
                            if (field.getType().toString().contains("Integer")) {
                                field.set(bean, Integer.parseInt(object.toString()));
                            }
                            if (field.getType().toString().contains("Float")) {
                                field.set(bean, Float.valueOf(object.toString()));
                            }
                            //处理LocalDateTime类型
                            /*if (field.getType().toString().contains("LocalDateTime")) {
                                field.set(bean, stringConvertLocalDateTime(object.toString()));
                            }*/
                        }
                    }
                    field.setAccessible(flag);
                }
            }
            return bean;
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
        return bean;
    }

    /**
     * 获取自身和上一级父类属性
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> Field[] getAllFields(Class<T> clazz){
        List<Field> fieldList = new ArrayList<>();
        fieldList.addAll(new ArrayList<>(Arrays.asList(clazz.getDeclaredFields())));
        if(clazz.getSuperclass()!=null){
            //写死的 取一层父类 TODO 没有好的解决方案
            fieldList.addAll(new ArrayList<>(Arrays.asList(clazz.getSuperclass().getDeclaredFields())));
        }
        Field[] fields = new Field[fieldList.size()];
        fieldList.toArray(fields);
        return fields;
    }

    public static void main(String[] args) {
        Map map = new HashMap();
        map.put("resName","1111");
        map.put("resType","123");
//        mapToBean(map,new OperationLog)
////        String indexName = map.get("indexName").toString();
////        String typeName = map.get("typeName").toString();
    }

    /**
     * 将List<T>转换为List<Map<String, Object>>
     * @param objList
     * @return
     *
     */
    public static <T> List<Map<String, Object>> objectsToMaps(List<T> objList) {
        List<Map<String, Object>> list = Lists.newArrayList();
        if (objList != null && objList.size() > 0) {
            Map<String, Object> map = null;
            T bean = null;
            for (int i = 0,size = objList.size(); i < size; i++) {
                bean = objList.get(i);
                map = beanToMap(bean);
                list.add(map);
            }
        }
        return list;
    }

    /**
     * 将List<Map<String,Object>>转换为List<T>
     * @param maps
     * @param clazz
     * @return
     * @throws InstantiationException
     * @throws IllegalAccessException
     */
    public static <T> List<T> mapsToObjects(List<Map<String, Object>> maps,Class<T> clazz) throws Exception {
        List<T> list = Lists.newArrayList();
        if (maps != null && maps.size() > 0) {
            Map<String, Object> map = null;
            T bean = null;
            for (int i = 0,size = maps.size(); i < size; i++) {
                map = maps.get(i);
                bean = mapToBean(map, clazz);
                list.add(bean);
            }
        }
        return list;
    }
}
