package net.wanji.common.utils.tool;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.*;
import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @ClassName ExcelUtils
 * @Description
 * @Author wanji
 * @Date 2020/6/30 13:22
 * @Version 1.0
 */
public class ExcelUtils {

    private static final String XLS = "xls";
    private static final String XLSX = "xlsx";
    private static final DateFormat FORMAT = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");

    /**
     * 输出数据到自定义模版的Excel输出流
     *
     * @param excelTemplate 自定义模版文件
     * @param data          数据
     * @param outputStream  Excel输出流
     * @param sheetName     sheet 名称
     * @param title         标题
     * @throws IOException 错误时抛出异常，由调用者处理
     */
    public static void writeDataToTemplateOutputStream(File excelTemplate,
                                                       List<Map<String, Object>> data,
                                                       OutputStream outputStream,
                                                       String sheetName,
                                                       List<String> title) throws IOException {
        Workbook book = ExcelUtils.getWorkbookFromExcel(excelTemplate);
        ExcelUtils.writeDataToWorkbook(title, data, book, sheetName);
        ExcelUtils.writeWorkbookToOutputStream(book, outputStream);
    }

    /**
     * 输出数据到自定义模版的Excel file
     *
     * @param excelTemplate 自定义模版文件
     * @param data          数据
     * @param sheetName     sheet 名称
     * @param title         标题
     * @throws IOException 错误时抛出异常，由调用者处理
     */
    public static void writeDataToTemplateFile(File excelTemplate,
                                               List<Map<String, Object>> data,
                                               String sheetName,
                                               List<String> title) throws IOException {
        Workbook book = ExcelUtils.getWorkbookFromExcel(excelTemplate);
        ExcelUtils.writeDataToWorkbook(title, data, book, sheetName);
        ExcelUtils.writeWorkbookToFile(book, excelTemplate);
    }

    /**
     * 从Excel文件获取Workbook对象
     *
     * @param excelFile Excel文件
     * @return Workbook对象
     * @throws IOException 错误时抛出异常，由调用者处理
     */
    public static Workbook getWorkbookFromExcel(File excelFile) throws IOException {
        try (
                InputStream inputStream = new FileInputStream(excelFile);
        ) {
            if (excelFile.getName().endsWith(XLS)) {
                return new HSSFWorkbook(inputStream);
            } else if (excelFile.getName().endsWith(XLSX)) {
                return new XSSFWorkbook(inputStream);
            } else {
                throw new IOException("文件类型错误");
            }
        }
    }

    /**
     * @return org.apache.poi.ss.usermodel.Sheet
     * @throws
     * @Author wanji
     * @Description 根据 workbook 和 sheetName 获取 sheet
     * @Date 14:10 2020/7/1
     * @Param [book, sheetName, title]
     */
    public static Sheet getSheet(Workbook book, String sheetName, List<String> title) {
        Sheet sheet = book.getSheet(sheetName);
        if (sheet == null) {
            sheet = book.createSheet(sheetName);
            Row row = sheet.createRow(0);
            // 设置表头
            for (int i = 0; i < title.size(); i++) {
                Cell cell = row.createCell(i);
                cell.setCellValue(title.get(i));
            }
        }
        return sheet;
    }

    /**
     * 输出数据到Workbook对象中指定 sheet
     *
     * @param title     标题，写在第一行，可传null
     * @param data      数据
     * @param book      Workbook对象
     * @param sheetName sheet页名称
     */
    public static void writeDataToWorkbook(List<String> title, List<Map<String, Object>> data, Workbook book, String sheetName) {
        Sheet sheet = ExcelUtils.getSheet(book, sheetName, title);
        int lastRowNum = sheet.getLastRowNum();
        for (int i = 0; i < data.size(); i++) {
            Map<String, Object> rowData = data.get(i);
            if (null == rowData) {
                continue;
            }
            ExcelUtils.makeRowAndCell(sheet, i, lastRowNum, title, rowData);
        }
    }

    /**
     * @return void
     * @throws
     * @Author wanji
     * @Description 组装 row 和 cell
     * @Date 14:22 2020/7/1
     * @Param [sheet, i, lastRowNum, title, rowData]
     */
    public static void makeRowAndCell(Sheet sheet, int i, int lastRowNum, List<String> title, Map<String, Object> rowData) {
        Row row = sheet.createRow(i + 1 + lastRowNum);
        for (int k = 0; k < title.size(); k++) {
            Cell cell = row.createCell(k);
            setValue(cell, rowData.get(title.get(k)));
        }
    }

    /**
     * 把Workbook对象内容输出到Excel文件
     *
     * @param book Workbook对象
     * @param file Excel文件
     * @throws FileNotFoundException 找不到文件异常，文件已创建，实际不存在该异常
     * @throws IOException           输入输出异常
     */
    public static void writeWorkbookToFile(Workbook book, File file) throws FileNotFoundException, IOException {
        /*if (!file.exists()) {
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
            }
            file.createNewFile();
        }*/
        try (
                OutputStream outputStream = new FileOutputStream(file);
        ) {
            writeWorkbookToOutputStream(book, outputStream);
        }
    }

    /**
     * 把Workbook对象输出到Excel输出流
     *
     * @param book         Workbook对象
     * @param outputStream Excel输出流
     * @throws IOException 错误时抛出异常，由调用者处理
     */
    public static void writeWorkbookToOutputStream(Workbook book, OutputStream outputStream) throws IOException {
        book.write(outputStream);
    }

    /**
     * 设置单元格值
     *
     * @param cell  单元格
     * @param value 值
     */
    public static void setValue(Cell cell, Object value) {
        if (null == cell) {
            return;
        }
        if (null == value) {
            cell.setCellValue((String) null);
        } else if (value instanceof Boolean) {
            cell.setCellValue((Boolean) value);
        } else if (value instanceof Date) {
            cell.setCellValue(FORMAT.format((Date) value));
        } else if (value instanceof Double) {
            cell.setCellValue((Double) value);
        } else {
            cell.setCellValue(value.toString());
        }
    }

    /**
     * 读取Excel
     *
     * @param pathname 文件路径名
     * @return 数据集合
     * @throws IOException 错误时抛出异常，由调用者处理
     */
    public static List<Map<String, Object>> readExcelWithSheet(String pathname, String sheetName) throws IOException {
        File file = new File(pathname);
        return readExcelWithSheet(file, sheetName);
    }

    /**
     * 读取Excel文件
     *
     * @param file Excel文件
     * @return 数据集合
     * @throws IOException 错误时抛出异常，由调用者处理
     */
    public static List<Map<String, Object>> readExcelWithSheet(File file, String sheetName) throws IOException {
        try (
                InputStream inputStream = new FileInputStream(file);
        ) {
            if (file.getName().endsWith(XLS)) {
                return readXlsWithSheet(inputStream, sheetName);
            } else if (file.getName().endsWith(XLSX)) {
                return readXlsxWithSheet(inputStream, sheetName);
            } else {
                throw new IOException("文件类型错误");
            }
        }
    }

    /**
     * 读取xls格式Excel文件
     *
     * @param inputStream Excel文件输入流
     * @return 数据集合
     * @throws IOException 错误时抛出异常，由调用者处理
     */
    public static List<Map<String, Object>> readXlsWithSheet(InputStream inputStream, String sheetName) throws IOException {
        Workbook workbook = new HSSFWorkbook(inputStream);
        return readExcelWithSheet(workbook, sheetName);
    }

    /**
     * @param inputStream 输入流
     * @param sheetName   sheet页名称
     * @return java.util.List<java.util.Map                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               <                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               java.lang.String                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               ,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               java.lang.Object>> map中key为表头，value为值
     * @Description: 根据sheetname 读取excel内容，默认按照xlsx格式读取，失败后读取xls
     * @Author Jack
     * @Date 2020/7/1 12:27
     **/
    public static List<Map<String, Object>> readXlsOrXlsxWithSheet(InputStream inputStream, String sheetName) throws IOException {
        try {
            return readXlsxWithSheet(inputStream, sheetName);
        } catch (Exception e) {
            return readXlsWithSheet(inputStream, sheetName);
        }
    }

    public static Workbook readXlsOrXlsx(InputStream inputStream) throws IOException {
        try {
            return new XSSFWorkbook(inputStream);
        } catch (Exception e) {
            return new HSSFWorkbook(inputStream);
        }
    }

    public static Workbook readXlsOrXlsx(File file) throws IOException {
        InputStream inputStream = new FileInputStream(file);
        try {
            return new XSSFWorkbook(inputStream);
        } catch (Exception e) {
            return new HSSFWorkbook(inputStream);
        } finally {
            inputStream.close();
        }
    }

    /**
     * 读取xlsx格式Excel文件
     *
     * @param inputStream Excel文件输入流
     * @return 数据集合
     * @throws IOException 错误时抛出异常，由调用者处理
     */
    public static List<Map<String, Object>> readXlsxWithSheet(InputStream inputStream, String sheetName) throws IOException {
        Workbook workbook = new XSSFWorkbook(inputStream);
        return readExcelWithSheet(workbook, sheetName);
    }

    /**
     * 读取Workbook
     *
     * @param book Workbook对象
     * @return 数据集合
     */
    public static List<Map<String, Object>> readExcelWithSheet(Workbook book, String sheetName) {
        return readExcelContent(book, sheetName);
    }

    /**
     * 读取指定页面的Excel
     *
     * @param book      Workbook对象
     * @param sheetName sheet 名称
     * @return 指定页面数据集合
     */
    public static List<Map<String, Object>> readExcelContent(Workbook book, String sheetName) {
        Sheet sheet = book.getSheet(sheetName);
        return readExcelContentBySheet(sheet);
    }

    /**
     * 读取指定页面的Excel
     *
     * @param sheet Sheet 对象
     * @return 指定页面数据集合
     */
    public static List<Map<String, Object>> readExcelContentBySheet(Sheet sheet) {
        List<Map<String, Object>> list = new ArrayList<>();
        // 标题
        Row row = sheet.getRow(0);
        List<String> title = new ArrayList<>();
        for (int j = 0; j < row.getLastCellNum(); j++) {
            Cell cell = row.getCell(j);
            title.add((String) getValue(cell));
        }

        for (int i = 1; i <= sheet.getLastRowNum(); i++) {
            row = sheet.getRow(i);
            // 如果当前行为空，则加入空，保持行号一致
            if (null == row) {
                list.add(null);
                continue;
            }
            Map<String, Object> columns = new HashMap<>();
            for (int j = 0; j < title.size(); j++) {
                Cell cell = row.getCell(j);
                columns.put(title.get(j), getValue(cell));
            }
            list.add(columns);
        }
        return list;
    }

    /**
     * 读取指定页面的Excel  Header
     *
     * @param sheetName sheetName
     * @return 指定页面Header
     */
    public static List<String> readExcelHeaderBySheet(Workbook book,String sheetName) {
        List<String> list = new ArrayList<>();
        Sheet sheet = book.getSheet(sheetName);
        // Header
        Row row = sheet.getRow(0);
        for (int j = 0; j < row.getLastCellNum(); j++) {
            Cell cell = row.getCell(j);
            list.add((String) getValue(cell));
        }
        return list;
    }

    /**
     * 解析单元格中的值
     *
     * @param cell 单元格
     * @return 单元格内的值
     */
    private static Object getValue(Cell cell) {
        if (null == cell) {
            return null;
        }
        Object value = null;
        switch (cell.getCellType()) {
            case BOOLEAN:
                value = cell.getBooleanCellValue();
                break;
            case NUMERIC:
                // 日期类型，转换为日期
                if (org.apache.poi.ss.usermodel.DateUtil.isCellDateFormatted(cell)) {
                    value = cell.getDateCellValue();
                } else { // 数值类型
                    // 默认返回double，创建BigDecimal返回准确值
                    value = new BigDecimal(cell.getNumericCellValue());
                }
                break;
            default:
                value = cell.toString();
                break;
        }
        return value;
    }

    public static void main(String[] args) {
        // Excel 文件
        File file = new File("D:\\test.xlsx");
        String sheetName = "测试";
        // 测试 写入
//        writeDataToTemplateFileDemo(file, sheetName);
        // 测试 读取
        List<Map<String, Object>> maps = readExcelWithSheetDemo(file, sheetName);
        System.out.println(maps);
    }

    private static List<Map<String, Object>> readExcelWithSheetDemo(File file, String sheetName) {
        List<Map<String, Object>> maps = null;
        try {
            maps = ExcelUtils.readExcelWithSheet(file, sheetName);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return maps;
    }

    private static void writeDataToTemplateFileDemo(File file, String sheetName) {

        // 数据
        List<Map<String, Object>> data = new ArrayList<Map<String, Object>>() {{
            add(new HashMap<String, Object>() {{
                put("id", 1);
                put("name", "张三");
            }});
            add(new HashMap<String, Object>() {{
                put("id", 2);
                put("name", "lisi");
            }});
            add(new HashMap<String, Object>() {{
                put("id", 3);
                put("name", 111);
            }});
        }};
        List<String> title = new ArrayList<String>() {{
            add("id");
            add("name");
        }};
        try {
            ExcelUtils.writeDataToTemplateFile(file, data, sheetName, title);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
