package net.wanji.common.utils.tool;


import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import javax.servlet.ServletInputStream;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.Calendar;

/**
 * Web上下文
 *
 * @author peiyuan.cai
 */
@Slf4j
public class WebCxt {

    private static final String CONTENT_TYPE = "application/json; charset=utf-8";
    private static final String CONTENT_TYPE_FOR_IE = "text/html; charset=utf-8";
    private static Calendar calendar = Calendar.getInstance();

    static {
        calendar.set(2099, 11, 31);
    }

    private static final long DATE = System.currentTimeMillis();
    public static final String TILE_CONTENT_TYPE = "application/x-protobuf";
    public static final long CACHE_TIME = 2592000L;
    public static final long MODIFY_DATE = System.currentTimeMillis();
    public static final String TILE_HEADER_ETAG = String.valueOf(MODIFY_DATE);
    public static final long EXPIRES_TIME = calendar.getTimeInMillis();


    public static final String CHECK_SQL = "^(.+)\\sand\\s(.+)|(.+)\\sor(.+)\\s$";
    public static final String CHECK_Q = "^.*'.*$";


    /**
     * 向response对象输出原始byte数据
     *
     * @param response
     * @param data
     */
    public static void renderRaw(HttpServletResponse response, byte[] data) {
        renderRaw(response, data, TILE_CONTENT_TYPE);
    }

    public static void renderRaw(HttpServletResponse response, byte[] data, String contentType) {
        ServletOutputStream out = null;
        try {
            setCacheHeader(response, String.valueOf(data.length));
            response.setContentType(contentType);
            response.setContentLength(data.length);
            out = response.getOutputStream();
            ByteArrayInputStream bais = new ByteArrayInputStream(data);
            byte[] buffer = new byte[1024];
            for (int len = -1; (len = bais.read(buffer)) != -1; ) {
                out.write(buffer, 0, len);
            }
            out.close();
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }
    }


    /**
     * 向response输出json字符串
     *
     * @param response
     * @param jsonText
     * @param forIE
     */
    public static void renderJson(HttpServletResponse response, String jsonText, boolean forIE) {

        PrintWriter writer = null;
        try {
            setCacheHeader(response, String.valueOf(jsonText.length()));
            response.setContentType(forIE ? CONTENT_TYPE_FOR_IE : CONTENT_TYPE);
            writer = response.getWriter();
            writer.write(jsonText);
            writer.flush();
        } catch (IOException e) {
        }
    }

    /**
     * 输出图片资源到response
     *
     * @param response
     * @param data
     */
    public static void renderImg(HttpServletResponse response, byte[] data) {
        ServletOutputStream out = null;
        try {
            setCacheHeader(response, String.valueOf(data.length));
            response.setContentType("image/png");
            out = response.getOutputStream();
            out.write(data);
            out.close();
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 输出文本到response
     *
     * @param response
     * @param text
     */
    public static void renderText(HttpServletResponse response, String text, String contentType) {
        try {
            setCacheHeader(response, String.valueOf(text.length()));
            response.setContentType(contentType);
            PrintWriter writer = response.getWriter();
            writer.print(text);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 设置response缓存头标识
     *
     * @param response
     */
    public static void setCorsHeader(HttpServletRequest request, HttpServletResponse response) {
        String url = request.getHeader("Origin");
        String Credentials = request.getHeader("Access-Control-Allow-Credentials");
        // log.debug("Access-Control-Allow-Origin:" + url);
        if (!StringUtils.isEmpty(url)) {
            // String val = response.getHeader("Access-Control-Allow-Origin");
            response.addHeader("Access-Control-Allow-Origin", url);
            response.addHeader("Access-Control-Allow-Credentials", "true");
	          /*  if (StringUtils.isEmpty(val)) {
	            }*/
        } else {
            response.setHeader("Access-Control-Allow-Origin", "*");
        }
    }

    /**
     * 设置response缓存头标识
     *
     * @param response
     */
    public static void setCacheHeader(HttpServletResponse response, String etag) {
        //response.setHeader("Access-Control-Allow-Origin", "*");

        response.setHeader("Cache-Control", "max-age=690000");
        response.setHeader("Connection", "keep-alive");
        response.setHeader("ETag", etag);
        response.setDateHeader("Expires", System.currentTimeMillis());
        response.setDateHeader("Last-Modified", System.currentTimeMillis());
        response.setDateHeader("Date", System.currentTimeMillis());
    }


    // 字符串读取
    // 方法一
    public static String ReadAsChars(HttpServletRequest request) {

        BufferedReader br = null;
        StringBuilder sb = new StringBuilder("");
        try {
            br = request.getReader();
            String str;
            while ((str = br.readLine()) != null) {
                sb.append(str);
            }
            br.close();
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        } finally {
            if (null != br) {
                try {
                    br.close();
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                }
            }
        }
        return sb.toString();
    }

    // 方法二
    public static void ReadAsChars2(HttpServletRequest request) {
        InputStream is = null;
        try {
            is = request.getInputStream();
            StringBuilder sb = new StringBuilder();
            byte[] b = new byte[4096];
            for (int n; (n = is.read(b)) != -1; ) {
                sb.append(new String(b, 0, n));
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        } finally {
            if (null != is) {
                try {
                    is.close();
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                }
            }
        }

    }

    // 二进制读取
    public static byte[] readAsBytes(HttpServletRequest request) {

        int len = request.getContentLength();
        byte[] buffer = new byte[len];
        ServletInputStream in = null;

        try {
            in = request.getInputStream();
            in.read(buffer, 0, len);
            in.close();
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        } finally {
            if (null != in) {
                try {
                    in.close();
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                }
            }
        }
        return buffer;
    }


    /**
     * 将文件以字节码方式返回到客户端
     *
     * @param response
     * @param cacheFile
     * @param s
     */
    public static void renderFileToRaw(HttpServletResponse response, File cacheFile, String s) {
        try (
                OutputStream bos = response.getOutputStream();
                FileInputStream fis = new FileInputStream(cacheFile);
        ) {
            byte[] b = new byte[1024];
            int n;
            while ((n = fis.read(b)) != -1) {
                bos.write(b, 0, n);
            }
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {

        } finally {

        }
    }
}
